//void PlotV0(const Char_t *fname = "jsf.pfoanl.zpole.thin.hitmerge.sub2ndary.4k.lockalld.emwithe+e-.ev.root")
//void PlotV0(const Char_t *fname = "jsf.pfoanl.zpole.thin.hitmerge.subonlyn.4k.lockalld.emwithe+e-.ev.root")
//void PlotV0(const Char_t *fname = "jsf.pfoanl.zpole.thin.hitmerge.subonlyn.4k.lockalld.emwithe+e-.ev.kmassfixed.v0.fixedjsfvertexing.root")
//void PlotV0(const Char_t *fname = "jsf.pfoanl.uds350.v0.std.10k.root")
//void PlotV0(const Char_t *fname = "jsf.pfoanl.uds91.v0.kinkdau.10k.root")
void PlotV0(const Char_t *fname = "jsf.pfoanl.uds91.v0.kinkmom.10k.root")
{
  // ------------------------------------------------------------
  //  Open shared libraries
  // ------------------------------------------------------------
  gSystem->Load("libJSF.so");
  gSystem->Load("libS4Utils.so");
  gSystem->Load("libAnlib.so");

  // ------------------------------------------------------------
  //  Open event file and apply cuts
  // ------------------------------------------------------------
  TFile *filep = TFile::Open(fname);
  TNtupleD *tupep = (TNtupleD *)gROOT->FindObject("hv0");

  //tupep->Draw(">>elist","npj1>=5&&npj2>=5&&abs(cosj1)<0.8&&abs(cosj2)<0.8&&mass>70&&mass<110","goff");
  tupep->Draw(">>elist","npj1>=5&&npj2>=5&&abs(cosj1)<0.8&&abs(cosj2)<0.8","goff");
  TEventList *elistp = (TEventList *)gROOT->FindObject("elist");

  const Double_t kNsig       = 2.0;	// fit range for gaussian fit in units of sigma
  const Double_t kQualityCut = 0.1;	// quality cut

  // ------------------------------------------------------------
  //  Define histograms
  // ------------------------------------------------------------
  TH1D *hQualp  = new TH1D("hQual" , "", 100, 0.00,  0.10);
  TH1D *hPairp  = new TH1D("hPair" , "", 100, 0.00,  0.10);
  TH1D *hK0Sp   = new TH1D("hK0S"  , "", 100, 0.48,  0.51);
  TH1D *hLamp   = new TH1D("hLam"  , "", 100, 1.11,  1.12);
  TH1D *hDLK0Sp = new TH1D("hDLK0S", "",  50, 0.00, 50.00);
  TH1D *hDLLamp = new TH1D("hDLLam", "",  50, 0.00, 50.00);
  // ------------------------------------------------------------
  //  Loop over selected events and fill up histograms
  // ------------------------------------------------------------
  Int_t nevt = elistp->GetN();
  cerr << "Nevt = " << nevt << endl;

  for (Int_t i=0; i<nevt; i++) {
    Int_t ievt = elistp->GetEntry(i);	// get event number

    Double_t npj1, npj2, cosj1, cosj2;
    Double_t pid, qual, e, px, py, pz, vx, vy, vz;

    tupep->SetBranchAddress("npj1" ,&npj1 );
    tupep->SetBranchAddress("npj2" ,&npj2 );
    tupep->SetBranchAddress("cosj1",&cosj1);
    tupep->SetBranchAddress("cosj2",&cosj2);
    tupep->SetBranchAddress("pid"  ,&pid  );
    tupep->SetBranchAddress("qual" ,&qual );
    tupep->SetBranchAddress("e"    ,&e    );
    tupep->SetBranchAddress("px"   ,&px   );
    tupep->SetBranchAddress("py"   ,&py   );
    tupep->SetBranchAddress("pz"   ,&pz   );
    tupep->SetBranchAddress("vx"   ,&vx   );
    tupep->SetBranchAddress("vy"   ,&vy   );
    tupep->SetBranchAddress("vz"   ,&vz   );

    tupep->GetEntry(ievt);	// get this event

    hQualp->Fill(qual, 1.);
    if (qual < kQualityCut) {
      Double_t mass = TMath::Sqrt(e*e-px*px-py*py-pz*pz);
      Double_t dlen = TMath::Sqrt(vx*vx+vy*vy+vz*vz);
      Double_t apid = TMath::Abs(pid);
      if      (apid < 100) {
        hPairp->Fill(mass, 1.); 
      } else if (apid > 100 && apid < 1000) {
        hK0Sp  ->Fill(mass, 1.); 
        hDLK0Sp->Fill(dlen, 1.);
      } else {
        hLamp  ->Fill(mass, 1.);
        hDLLamp->Fill(dlen, 1.);
      }
    }
  }
  // ------------------------------------------------------------
  //  Draw histograms
  // ------------------------------------------------------------
  gStyle->SetOptFit(1);
  gStyle->SetOptStat(111111);

  TCanvas *c1p = new TCanvas("c1","",20,20,1220,820);
  TCanvas &c1  = *c1p;
  c1.Divide(3,2);
  Int_t id = 0;

  TPad *cpadp;

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hPairp->Draw();
  hPairp->SetFillColor(42);
  hPairp->SetFillStyle(1000);
  hPairp->GetXaxis()->SetTitle("Mass [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hK0Sp->Draw();
  hK0Sp->SetFillColor(42);
  hK0Sp->SetFillStyle(1000);
  hK0Sp->GetXaxis()->SetTitle("Mass [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hLamp->Draw();
  hLamp->SetFillColor(42);
  hLamp->SetFillStyle(1000);
  hLamp->GetXaxis()->SetTitle("Mass [GeV]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hQualp->Draw();
  hQualp->SetFillColor(42);
  hQualp->SetFillStyle(1000);
  hQualp->GetXaxis()->SetTitle("Quality [cm]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDLK0Sp->Draw();
  hDLK0Sp->SetFillColor(42);
  hDLK0Sp->SetFillStyle(1000);
  hDLK0Sp->GetXaxis()->SetTitle("Decay Length [cm]");

  id++; cpadp = (TPad *)c1.cd(id);
  cpadp->SetFillColor(30);
  cpadp->SetFrameFillColor(TColor::GetColor("#fbf7ca"));
  hDLLamp->Draw();
  hDLLamp->SetFillColor(42);
  hDLLamp->SetFillStyle(1000);
  hDLLamp->GetXaxis()->SetTitle("Decay Length [cm]");

  c1.cd();
}
