//
// tpcanl.C
//
// 


void tpcanl()
{
//----------------------------------
// Reset ROOT
//----------------------------------

  gROOT->Reset();

  gErrorAbortLevel = kWarning + 1; 
  gRandom->SetSeed();
       
//----------------------------------
// Create JSF runtime environment
//----------------------------------

  JSFSteer *jsfp = new JSFSteer();
    
//----------------------------------
// Load user defined classes
//----------------------------------

  gSystem->Load("libS4Kalman.so");  
  gSystem->Load("libS4Geom.so");  
  gSystem->Load("libS4KalTrack.so");  
  gSystem->Load("libS4GENMCTruth.so");
  gSystem->Load("libS4HISTMCTruth.so");
  gSystem->Load("libU4VHitMaker.so");
  gSystem->Load("libU4KalDetector.so");  
  gSystem->Load("libU4TPCKalDetector.so");  
  gSystem->Load("libU4TPCHitMaker.so");
  gSystem->Load("libS4VTXMCTruth.so");
  gSystem->Load("libS4ITMCTruth.so");
  gSystem->Load("libS4TPCMCTruth.so");
  gSystem->Load("libS4TPCHitMaker.so");
  gSystem->Load("libU4VTrackMaker.so");  
  gSystem->Load("libS4Kern.so");
  gSystem->Load("libU4TPCTrackMaker.so");  
  gSystem->Load("libS4TPCTrackMaker.so");  
  gSystem->Load("libS4TPCAnalysis.so");

//----------------------------------
// Set output file
//----------------------------------

  TFile *foutp = new TFile("jsf.tpcanl.root","RECREATE");
  TFile &fout  = *foutp;
  jsfp->SetOutput(fout);

//----------------------------------
// Set input file
//----------------------------------
#if 0
  S4VMCTruth::SetInputFileName("hit.dat");
#else
  TFile file("jsfj4.tpc.root");
  file.ls();
  jsfp->SetInput(file);
#endif
  Int_t nevent=jsfp->Env()->GetValue("JSFSteer.Nevent",999999);  
  Int_t minevt=1;
  Int_t maxevt=minevt+nevent;

//----------------------------------
// Insert modules
//----------------------------------

  JSFJ4BufManager  *bufman      = new JSFJ4BufManager();

  S4GENMCTruth     *mctruthgen  = new S4GENMCTruth();
  S4HISTMCTruth    *mctruthhist = new S4HISTMCTruth();
  S4VTXMCTruth     *vtxmctruth  = new S4VTXMCTruth();
  S4ITMCTruth      *itmctruth   = new S4ITMCTruth();
  S4TPCMCTruth     *tpcmctruth  = new S4TPCMCTruth();
  S4TPCHitMaker    *hitmaker    = new S4TPCHitMaker();
  S4TPCTrackMaker  *trackmaker  = new S4TPCTrackMaker();
  S4TPCAnalysis    *analysis    = new S4TPCAnalysis();

  bufman    ->SetMakeBranch(kFALSE);
  tpcmctruth->SetMakeBranch(kFALSE);
  hitmaker  ->SetMakeBranch(kFALSE);
  trackmaker->SetMakeBranch(kFALSE);
  analysis  ->SetMakeBranch(kFALSE);

//----------------------------------
// set parameters
//----------------------------------
   S4TPCHit::SetSpatialResolution("experimental"); // flat or experimental
   S4TPCHit::SetT0offset(7.);              // nsec
   //S4TPCHitMakerBuf::SwitchOnMergeHit();
   //S4TPCHitMakerBuf::SetMergeRange(1., 5.);

//----------------------------------
// Job initialization
//----------------------------------
 
  jsfp->Initialize();

//----------------------------------
// Define KalDetector
//----------------------------------

  TKalDetCradle    *cradle = new TKalDetCradle();
  U4TPCKalDetector *tpcdet = new U4TPCKalDetector("FillLayer");
  trackmaker->SetDetector(*tpcdet);

  cradle->Install(*tpcdet);
  cradle->Close();
  //cradle->SwitchOffMS();
  //cradle->SwitchOffDEDX();

//----------------------------------
// Start event loop
//----------------------------------

  for(Int_t ev=minevt;ev<=maxevt;ev++){
    if (!jsfp->GetEvent(ev)) break; 
    cerr << " ---- Event " << jsfp->GetEventNumber() << " ----" << endl;
    if (!jsfp->Process(ev)) {
      jsfp->FillTree();
      jsfp->Clear();
      break;
    }
    //gObjectTable->Print();
    jsfp->FillTree();
    jsfp->Clear();
  }
  jsfp->Terminate();
  fout.Write();
}
