///////////////////////////////////////////////////////////////////
//
//  Convert J4Hits to S4Exact hits
//
//
//////////////////////////////////////////////////////////////////
#ifdef __HasJSFFromLCIO__
#include "J4IRBCALHit.hh"
#include "S4BCALExactHit.h"

// ------------------------------------------------------------------
S4BCALExactHit::S4BCALExactHit(J4IRBCALHit *ahit)
              : S4VExactHit(ahit->GetPreHitID(),        // hit ID
                            ahit->GetTrackID(),         // track ID
                            ahit->GetMotherTrackID(),   // mother track ID
                            ahit->GetPDGEncoding(),     // PDG code
                            ahit->GetCharge(),          // charge
                         TVector3(ahit->GetXcm().x() // E-weighted hit pos
                                /ahit->GetEnergyDeposit()/cm,
    		                     ahit->GetXcm().y()
                                        /ahit->GetEnergyDeposit()/cm,
    		                     ahit->GetXcm().z()
                                        /ahit->GetEnergyDeposit()/cm),
                            ahit->GetTof()/nanosecond,  // TOF
                            TVector3(ahit->GetMomentum().x(),
				     ahit->GetMomentum().y(),
				     ahit->GetMomentum().z()),   // momentum
                            ahit->GetTotalEnergy(),             // energy
                            ahit->GetEnergyDeposit()/GeV, // energy deposit
                            0),                         // address ptr 
		//                fPreHitID(ahit->GetPreHitID()),         // prehit ID
                fPreHitID(ahit->GetPreHitID()),     // prehit ID
                fCellID(ahit->GetCellID()),             // cell ID
		//                fIsBarrel(ahit->IsBarrel()),            // is Barrel?
		//                fIsEM(ahit->IsEM()),                    // is EM?
		f1stHit(ahit->GetXhit().x()/cm, ahit->GetXhit().y()/cm, 
			ahit->GetXhit().z()/cm)  // 1st hit position
                // fPreHitPtr(0)
{
}

// ==================================================================
#include "J4IRBCALPreHit.hh"
#include "G4ParticleDefinition.hh"
#include "S4BCALPreHit.h"

//---------------------------------------------------------------------
S4BCALPreHit::S4BCALPreHit(J4IRBCALPreHit *p)
            : S4VExactHit(p->GetPreHitID(),                       // prehit ID
                          p->GetTrackID(),                        // track ID
                          0,                                      // mother track ID
                          p->GetPDGEncoding(),                    // PDG code
                          p->GetCharge(),                         // charge
                          TVector3(p->GetInjectionPoint().x()/cm, // position
                                   p->GetInjectionPoint().y()/cm,
                                   p->GetInjectionPoint().z()/cm),
                          p->GetTof()/nanosecond,                 // TOF
                          TVector3(p->GetMomentum().x()/GeV,      // momentum
                                   p->GetMomentum().y()/GeV,
                                   p->GetMomentum().z()/GeV),
#if 0
                          p->GetKineticEnergy()/GeV,              // energy
#else
                          p->GetTotalEnergy()/GeV,                // energy
#endif
                          0,                                      // edep
                          0),                                     // address ptr
              fParticleName(p->GetParticle()->GetParticleName()), // particle name
              fPostHitID(p->GetPostHitID())                       // post hit ID
{
}

// ==================================================================
#include "J4CALHit.hh"
#include "J4CLXHit.hh"
#include "S4CALExactHit.h"

// ------------------------------------------------------------------
S4CALExactHit::S4CALExactHit(J4CALHit *ahit)
              : S4VExactHit(0,                          // hit ID
                            ahit->GetTrackID(),         // track ID
                            ahit->GetMotherTrackID(),   // mother track ID
                            ahit->GetPDGEncoding(),     // PDG code
                            0,                          // charge
                            TVector3(ahit->GetXcm().x() // E-weighted hit pos
                                    /ahit->GetEnergyDeposit()/cm,
		                     ahit->GetXcm().y()
                                    /ahit->GetEnergyDeposit()/cm,
		                     ahit->GetXcm().z()
                                    /ahit->GetEnergyDeposit()/cm),
                            ahit->GetTof()/nanosecond,  // TOF
                            TVector3(0.,0.,0.),         // momentum
                            0,                          // energy
                            ahit->GetEnergyDeposit()/GeV, // energy deposit
                            0),                         // address ptr 
                fPreHitID(ahit->GetPreHitID()),         // prehit ID
                fCellID(ahit->GetCellID()),             // cell ID
                fIsBarrel(ahit->IsBarrel()),            // is Barrel?
                fIsEM(ahit->IsEM()),                    // is EM?
                fCellPosition(ahit->GetXcell().x()/cm,  // cell position
                              ahit->GetXcell().y()/cm,
			      ahit->GetXcell().z()/cm),
                fCellXcm(ahit->GetXcm().x()/(GeV*cm),   // E-weighted pos
		         ahit->GetXcm().y()/(GeV*cm),
		         ahit->GetXcm().z()/(GeV*cm)),
                fPreHitPtr(0)
{
}


// ------------------------------------------------------------------
S4CALExactHit::S4CALExactHit(J4CLXHit *ahit)
              : S4VExactHit(0,                          // hit ID
                            ahit->GetTrackID(),         // track ID
                            ahit->GetMotherTrackID(),   // mother track ID
                            ahit->GetPDGEncoding(),     // PDG code
                            ahit->GetCharge(),          // charge
                            TVector3(ahit->GetXcm().x() // E-weighted hit pos
                                    /ahit->GetEnergyDeposit()/cm,
		                     ahit->GetXcm().y()
                                    /ahit->GetEnergyDeposit()/cm,
		                     ahit->GetXcm().z()
                                    /ahit->GetEnergyDeposit()/cm),
                            ahit->GetTof()/nanosecond,  // TOF
                            TVector3(ahit->GetMomentum().x()/GeV, // momentum
				     ahit->GetMomentum().y()/GeV,
				     ahit->GetMomentum().z()/GeV ), 
                            0,                          // energy
                            ahit->GetEnergyDeposit()/GeV, // energy deposit
                            0),                         // address ptr 
                fPreHitID(ahit->GetPreHitID()),         // prehit ID
                fCellID(ahit->GetCellID()),             // cell ID
		fIsBarrel(ahit->IsBarrel()),            // is Barrel?
		fIsEM(ahit->IsEM()),                    // is EM?
		fCellPosition(ahit->GetXcell().x()/cm,  // cell position
		              ahit->GetXcell().y()/cm,
			      ahit->GetXcell().z()/cm),
                fCellXcm(ahit->GetXcm().x()/(GeV*cm),   // E-weighted pos
		         ahit->GetXcm().y()/(GeV*cm),
		         ahit->GetXcm().z()/(GeV*cm)),
                fPreHitPtr(0)
{
}

//====================================================================
#include "J4CALPreHit.hh"
#include "J4CLXPreHit.hh"
#include "G4ParticleDefinition.hh"
#include "S4CALPreHit.h"

//---------------------------------------------------------------------
S4CALPreHit::S4CALPreHit(J4CALPreHit *p)
            : S4VExactHit(p->GetPreHitID(),                       // prehit ID
                          p->GetTrackID(),                        // track ID
                          0,                                      // mother track ID
                          p->GetPDGEncoding(),                    // PDG code
                          p->GetCharge(),                         // charge
                          TVector3(p->GetInjectionPoint().x()/cm, // position
                                   p->GetInjectionPoint().y()/cm,
                                   p->GetInjectionPoint().z()/cm),
                          p->GetTof()/nanosecond,                 // TOF
                          TVector3(p->GetMomentum().x()/GeV,      // momentum
                                   p->GetMomentum().y()/GeV,
                                   p->GetMomentum().z()/GeV),
#if 0
                          p->GetKineticEnergy()/GeV,              // energy
#else
                          p->GetTotalEnergy()/GeV,                // energy
#endif
                          0,                                      // edep
                          0),                                     // address ptr
              fParticleName(p->GetParticle()->GetParticleName()), // particle name
              fPostHitID(p->GetPostHitID())                       // post hit ID
{
}

//---------------------------------------------------------------------
S4CALPreHit::S4CALPreHit(J4CLXPreHit *p)
            : S4VExactHit(p->GetPreHitID(),                       // prehit ID
                          p->GetTrackID(),                        // track ID
                          0,                                      // mother track ID
                          p->GetPDGEncoding(),                    // PDG code
                          p->GetCharge(),                         // charge
                          TVector3(p->GetInjectionPoint().x()/cm, // position
                                   p->GetInjectionPoint().y()/cm,
                                   p->GetInjectionPoint().z()/cm),
                          p->GetTof()/nanosecond,                 // TOF
                          TVector3(p->GetMomentum().x()/GeV,      // momentum
                                   p->GetMomentum().y()/GeV,
                                   p->GetMomentum().z()/GeV),
#if 0
                          p->GetKineticEnergy()/GeV,              // energy
#else
                          p->GetTotalEnergy()/GeV,                // energy
#endif
                          0,                                      // edep
                          0),                                     // address ptr
              fParticleName(p->GetParticle()->GetParticleName()), // particle name
              fPostHitID(p->GetPostHitID())                       // post hit ID
{
}

//====================================================================
#include "J4CALPostHit.hh"
#include "G4ParticleDefinition.hh"
#include "S4CALPostHit.h"
//---------------------------------------------------------------------
S4CALPostHit::S4CALPostHit(J4CALPostHit *p)
            : S4VExactHit(p->GetPostHitID(),                      // prehit ID
                          p->GetTrackID(),                        // track ID
                          0,                                      // mother track ID
                          p->GetPDGEncoding(),                    // PDG code
                          p->GetCharge(),                         // charge
                          TVector3(p->GetPrePosition().x()/cm, // position
                                   p->GetPrePosition().y()/cm,
                                   p->GetPrePosition().z()/cm),
                          p->GetTof()/nanosecond,                 // TOF
                          TVector3(p->GetMomentum().x()/GeV,      // momentum
                                   p->GetMomentum().y()/GeV,
                                   p->GetMomentum().z()/GeV),
#if 0
                          p->GetKineticEnergy()/GeV,              // energy
#else
                          p->GetTotalEnergy()/GeV,                // energy
#endif
                          0,                                      // edep
                          0),                                     // address ptr
              fParticleName(p->GetParticle()->GetParticleName()), // particle name
              fPreHitID(p->GetCALPreHitID())                    // pre hit ID
{
}

//====================================================================

#include "S4CDCExactHit.h"
#include "J4VCDCDriftRegion.hh"
#include "J4VCDCDriftRegionHit.hh"

#define __MMTOCM__   0.100
#define __MEVTOGEV__ 0.001

//=====================================================================
//* constructor -------------------------------------------------------

S4CDCExactHit::S4CDCExactHit(const J4VCDCDriftRegionHit *hitp)
{
    J4VCDCDriftRegion *cpt = (J4VCDCDriftRegion *)hitp->GetComponent();
    TVector3 pre  (hitp->GetPrePosition().x(),
                   hitp->GetPrePosition().y(),
                   hitp->GetPrePosition().z());
    TVector3 post (hitp->GetPostPosition().x(),
                   hitp->GetPostPosition().y(),
                   hitp->GetPostPosition().z());
    TVector3 mom  (hitp->GetMomentum().x(),
                   hitp->GetMomentum().y(),
                   hitp->GetMomentum().z());
    TVector3 wendm(hitp->GetWireEnd(0).x(),
                   hitp->GetWireEnd(0).y(),
                   hitp->GetWireEnd(0).z());
    TVector3 wendp(hitp->GetWireEnd(1).x(),
                   hitp->GetWireEnd(1).y(),
                   hitp->GetWireEnd(1).z());

   *this = S4CDCExactHit(0,                           // event No.
                         cpt->GetGlobalLayerNumber(), // layer No.
                         cpt->GetMyID(),              // wire No.
                         hitp->GetCloneID(),          // cell No.
                         hitp->GetTrackID(),          // track ID
                         hitp->GetMotherTrackID(),    // mother track ID
                         hitp->GetHitNumber(),        // hit No.
                         hitp->GetPDGEncoding(),      // PID
                         (Int_t)hitp->GetCharge(),    // charge
                         cpt->GetDeltaPhi(cpt),       // cell dphi
                         -0.5*cpt->GetDeltaPhi(cpt),  // cell start phi
              __MMTOCM__*pre,                         // pre position
              __MMTOCM__*post,                        // post position
            __MEVTOGEV__*mom,                         // momentum
            __MEVTOGEV__*hitp->GetTotalEnergy(),      // total energy
            __MEVTOGEV__*hitp->GetEnergyDeposit(),    // energy deposit
                         hitp->GetTof(),              // TOF
              __MMTOCM__*wendm,                       // -ve z end of wire
              __MMTOCM__*wendp,                       // +ve z end of wire
                         hitp->GetTanStereo(),        // tan(stereo)
              __MMTOCM__*hitp->GetRwaist());          // r_waist
}

//====================================================================
#include "G4Tubs.hh"
#include "J4CTLayerHit.hh"
#include "S4CTExactHit.h"

// ------------------------------------------------------------------
S4CTExactHit::S4CTExactHit(J4CTLayerHit *ahit)
{

  J4VComponent *vc=ahit->GetComponent();

  fWireNo=vc->GetMyID();
  fLayerNo=vc->GetMother()->GetMother()->GetMyID();
  fTrackID=ahit->GetTrackID();

  fPDGParticleID = ahit->GetParticle()->GetPDGEncoding();   
  

  fDeltaPhiAngle = ((G4Tubs*)vc->GetSolid())->GetDeltaPhiAngle();
  fStartPhiAngle = ((G4Tubs*)vc->GetSolid())->GetStartPhiAngle();

  G4ThreeVector pos  = ahit->GetPrePosition();

  Double_t ghphi    = pos.phi();
  while (ghphi < 0)      ghphi += 2*TMath::Pi();
  while (ghphi > 2*M_PI) ghphi -= 2*TMath::Pi();   
  Double_t cellphi = ghphi - fStartPhiAngle;
  while(cellphi < 0)      cellphi += 2*TMath::Pi();
  while(cellphi > 2*TMath::Pi()) cellphi -= 2*TMath::Pi();   
  fCellID  = (Int_t)(cellphi/fDeltaPhiAngle);
		
  fPrePosition=TVector3( ahit->GetPrePosition().x()/cm, ahit->GetPrePosition().y()/cm,
			 ahit->GetPrePosition().z()/cm);
  fPostPosition=TVector3( ahit->GetPostPosition().x()/cm, ahit->GetPostPosition().y()/cm,
			  ahit->GetPostPosition().z()/cm);
  fHitPosition=TVector3( pos.x()/cm, pos.y()/cm, 
			  pos.z()/cm);
  fMomentum=TVector3( ahit->GetMomentum().x()/GeV, ahit->GetMomentum().y()/GeV,
		      ahit->GetMomentum().z()/GeV);
  fEnergyDeposit=ahit->GetEnergyDeposit();
  fTof=ahit->GetTof();	


}

//====================================================================
#include "J4IRFCALHit.hh"
#include "S4FCALExactHit.h"

// ------------------------------------------------------------------
S4FCALExactHit::S4FCALExactHit(J4IRFCALHit *ahit)
              : S4VExactHit(ahit->GetPreHitID(),        // hit ID
                            ahit->GetTrackID(),         // track ID
                            ahit->GetMotherTrackID(),   // mother track ID
                            ahit->GetPDGEncoding(),     // PDG code
                            ahit->GetCharge(),          // charge
                         TVector3(ahit->GetXcm().x()/
				  ahit->GetEnergyDeposit()/cm, // E-weighted hit pos
				  ahit->GetXcm().y()/ahit->GetEnergyDeposit()/cm,
				  ahit->GetXcm().z()/ahit->GetEnergyDeposit()/cm),
                            ahit->GetTof()/nanosecond,  // TOF
			    TVector3(ahit->GetXcm().x()/
				    ahit->GetEnergyDeposit()/cm,   // E-weighted pos
				    ahit->GetXcm().y()/ahit->GetEnergyDeposit()/cm,
				    ahit->GetXcm().z()/ahit->GetEnergyDeposit()/cm),
                            ahit->GetTotalEnergy(),              // energy
                            ahit->GetEnergyDeposit()/GeV, // energy deposit
                            0),                         // address ptr 
		//                fPreHitID(ahit->GetPreHitID()),         // prehit ID
                fPreHitID(ahit->GetPreHitID()),         // prehit ID
                fCellID(ahit->GetCellID()),             // cell ID
		//                fIsBarrel(ahit->IsBarrel()),            // is Barrel?
		//                fIsEM(ahit->IsEM()),                    // is EM?
		f1stHit(ahit->GetXhit().x()/cm, ahit->GetXhit().y()/cm, 
			ahit->GetXhit().z()/cm) // 1st hit position
                // fPreHitPtr(0)
{
//  Print();

}

//====================================================================
#include "J4IRFCALPreHit.hh"
#include "G4ParticleDefinition.hh"
#include "S4FCALPreHit.h"
//---------------------------------------------------------------------
S4FCALPreHit::S4FCALPreHit(J4IRFCALPreHit *p)
            : S4VExactHit(p->GetPreHitID(),                       // prehit ID
                          p->GetTrackID(),                        // track ID
                          0,                                      // mother track ID
                          p->GetPDGEncoding(),                    // PDG code
                          p->GetCharge(),                         // charge
                          TVector3(p->GetInjectionPoint().x()/cm, // position
                                   p->GetInjectionPoint().y()/cm,
                                   p->GetInjectionPoint().z()/cm),
                          p->GetTof()/nanosecond,                 // TOF
                          TVector3(p->GetMomentum().x()/GeV,      // momentum
                                   p->GetMomentum().y()/GeV,
                                   p->GetMomentum().z()/GeV),
#if 0
                          p->GetKineticEnergy()/GeV,              // energy
#else
                          p->GetTotalEnergy()/GeV,                // energy
#endif
                          0,                                      // edep
                          0),                                     // address ptr
              fParticleName(p->GetParticle()->GetParticleName()), // particle name
              fPostHitID(p->GetPostHitID())                       // post hit ID
{
}

//====================================================================

#include "J4ITLayerHit.hh"
#include "S4ITExactHit.h"
#include "JSFSteer.h"
#include "JSFEnv.h"

//=====================================================================
S4ITExactHit::S4ITExactHit(const J4ITLayerHit *hitp)
            : S4VExactHit(0,                              // hit ID
                          hitp->GetTrackID(),             // track ID
                          hitp->GetMotherTrackID(),       // mother track ID
                          hitp->GetPDGEncoding(),         // particle ID
                          (Int_t)hitp->GetCharge(),       // charge
                          (1. / cm) * TVector3(hitp->GetPrePosition().x(),
                                               hitp->GetPrePosition().y(),
                                               hitp->GetPrePosition().z()),
                          hitp->GetTof(),                 // TOF
                          (1. / GeV) * TVector3(hitp->GetMomentum().x(),
                                                hitp->GetMomentum().y(),
                                                hitp->GetMomentum().z()),
                          hitp->GetTotalEnergy()   / GeV, // total energy
                          hitp->GetEnergyDeposit() / GeV, // energy deposit
                          0,                              // address ptr
                          0),                             // event ID 
              fLayerID(hitp->GetComponent()->GetMyID()), // pad raw No.
              fPrePosition((1./cm)*TVector3(hitp->GetPrePosition().x(),
                                            hitp->GetPrePosition().y(),
                                            hitp->GetPrePosition().z())),
              fPostPosition((1./cm)*TVector3(hitp->GetPostPosition().x(),
                                             hitp->GetPostPosition().y(),
                                             hitp->GetPostPosition().z()))
{
   TVector3 hitpos  = GetPosition();
   fLayerR          = hitpos.Pt();
   fPhi             = hitpos.Phi();
   static Int_t nb  = gJSF->Env()->GetValue("J4IT.NBarrel", 4);
   static Int_t ne  = gJSF->Env()->GetValue("J4IT.NEndcap", 7);
   fSide            = fLayerID < nb ? 0 : (fLayerID < ne ? 1 : -1);
}

//====================================================================

#include "J4VTXPixelHit.hh"
#include "J4VTXLayerHit.hh"
#include "S4VTXExactHit.h"

//=====================================================================
S4VTXExactHit::S4VTXExactHit(J4VTXPixelHit *ahit)
             : S4VExactHit(0,                            // hit ID
                           ahit->GetTrackID(),           // track ID
                           ahit->GetMotherTrackID(),     // mother track ID
                           ahit->GetPDGEncoding(),       // particle ID
                           (Int_t)ahit->GetCharge(),     // charge
                           TVector3(ahit->GetInPosition().x()/cm, 
                                           ahit->GetInPosition().y()/cm, 
                                           ahit->GetInPosition().z()/cm),
                           ahit->GetTof(),               // TOF
			   TVector3(ahit->GetPMomentum().x()/GeV, 
				    ahit->GetPMomentum().y()/GeV, 
				    ahit->GetPMomentum().z()/GeV),
                           ahit->GetTotalEnergy()/GeV,   // total energy
                           ahit->GetEnergyDeposit()/GeV, // energy deposit
                           0,                            // address ptr
                           0),                           // event ID 
               fPrePosition(TVector3(ahit->GetInPosition().x()/cm,
                                     ahit->GetInPosition().y()/cm,
                                     ahit->GetInPosition().z()/cm)),
               fPostPosition(TVector3(ahit->GetOutPosition().x()/cm,
                                      ahit->GetOutPosition().y()/cm,
                                      ahit->GetOutPosition().z()/cm)),
               fLayerID(ahit->GetLayerID()),          // Layer ID of Hit
	       fLadderID(ahit->GetLadderID()),        // Lader ID
	       fSensorID(ahit->GetSensorID()),        // Sensor ID
	       fPixelThetaID(ahit->GetPixThetaID()),  // Theta ID of pixel
	       fPixelPhiID(ahit->GetPixPhiID())       // PhiID of pixel
{
   fPhi = GetPosition().Phi();
   fZ   = GetPosition().Z();
   fR   = GetPosition().Pt();
}

//=====================================================================
S4VTXExactHit::S4VTXExactHit(J4VTXLayerHit *ahit)
             : S4VExactHit(0,                            // hit ID
                           ahit->GetTrackID(),           // track ID
                           ahit->GetMotherTrackID(),     // mother track ID
                           ahit->GetPDGEncoding(),       // particle ID
                           (Int_t)ahit->GetCharge(),     // charge
			   TVector3(ahit->GetInPosition().x()/cm, 
				         ahit->GetInPosition().y()/cm, 
				         ahit->GetInPosition().z()/cm),
                           ahit->GetTof(),               // TOF
			   TVector3(ahit->GetMomentum().x()/GeV, 
				    ahit->GetMomentum().y()/GeV, 
				    ahit->GetMomentum().z()/GeV),
                           ahit->GetTotalEnergy()/GeV,   // total energy
                           ahit->GetEnergyDeposit()/GeV, // energy deposit
                           0,                            // address ptr
                           0),                           // event ID 
               fPrePosition(TVector3(ahit->GetInPosition().x()/cm,
                                     ahit->GetInPosition().y()/cm,
                                     ahit->GetInPosition().z()/cm)),
               fPostPosition(TVector3(ahit->GetOutPosition().x()/cm,
                                      ahit->GetOutPosition().y()/cm,
                                      ahit->GetOutPosition().z()/cm)),
               fLayerID(ahit->GetLayerID()), // Layer ID of Hit
	       fLadderID(0),                 // Lader ID
	       fSensorID(0),                 // Sensor ID
	       fPixelThetaID(0),             // Theta ID of pixel
	       fPixelPhiID(0)                // PhiID of pixel
{
   fPhi = GetPosition().Phi();
   fZ   = GetPosition().Z();
   fR   = GetPosition().Pt();
}


//====================================================================
#include "J4TPCLayerHit.hh"
#include "S4TPCExactHit.h"

S4TPCExactHit::S4TPCExactHit(const J4TPCLayerHit *hitp)
             : S4VExactHit(0,                              // hit ID
                           hitp->GetTrackID(),             // track ID
                           hitp->GetMotherTrackID(),       // mother track ID
                           hitp->GetPDGEncoding(),         // particle ID
                           hitp->GetCharge(),              // charge
                           TVector3(0.,0.,0.),             // position
                           hitp->GetTof(),                 // TOF
                           (1 / GeV) * TVector3(hitp->GetMomentum().x(),
                                                hitp->GetMomentum().y(),
                                                hitp->GetMomentum().z()),
                           hitp->GetTotalEnergy()   / GeV, // total energy
                           hitp->GetEnergyDeposit() / GeV, // energy deposit
                           0,                              // address ptr
                           0),                             // event ID 
               fPadRowNumber(hitp->GetComponent()->GetMyID()), // pad raw No.
               fPrePosition((1/cm)*TVector3(hitp->GetPrePosition().x(),
                                            hitp->GetPrePosition().y(),
                                            hitp->GetPrePosition().z())),
               fPostPosition((1/cm)*TVector3(hitp->GetPostPosition().x(),
                                             hitp->GetPostPosition().y(),
                                             hitp->GetPostPosition().z()))
{
   JSFEnv   &env  = *gJSF->Env();
   static const Double_t halfz = env.GetValue("J4.TPC.HalfZ", 260.)
                               - env.GetValue("J4TPC.Endcap.HalfThickness", 1.) * 2
                               - env.GetValue("J4TPC.PadPlane.HalfThickness", 1.5) * 2;
   TVector3 pos = (fPrePosition.Pt() > fPostPosition.Pt()) ? fPostPosition
                                                           : fPrePosition;
   SetPosition(pos);
   Double_t z    = pos.Z();
   fZdrift       = z > 0 ? halfz - z : halfz + z;
   fPadRowRadius = pos.Pt();
   fPhi          = pos.Phi();
   fSide         = z > 0 ? 1 : -1;
}

//====================================================================
#include "J4TPCPostHit.hh"
#include "S4TPCPostHit.h"

S4TPCPostHit::S4TPCPostHit(const J4TPCPostHit *hitp)
            : S4VExactHit(hitp->GetPostHitID(),           // hit ID
                          hitp->GetTrackID(),             // track ID
                          hitp->GetMotherTrackID(),       // mother track ID
                          hitp->GetPDGEncoding(),         // particle ID
                          (Int_t)hitp->GetCharge(),       // charge
                          (1. / cm) * (TVector3(hitp->GetPrePosition().x(),
                                                hitp->GetPrePosition().y(),
                                                hitp->GetPrePosition().z())),
                          hitp->GetTof(),                 // TOF
                          (1 / GeV) * TVector3(hitp->GetMomentum().x(),
                                               hitp->GetMomentum().y(),
                                               hitp->GetMomentum().z()),
                          hitp->GetTotalEnergy()   / GeV, // total energy
                          0.,                             // energy deposit
                          0,                              // address ptr
                          0)                              // event ID 
{
}

//====================================================================
#include "J4MUDHit.hh"
#include "S4MUDExactHit.h"

// ------------------------------------------------------------------
S4MUDExactHit::S4MUDExactHit(J4MUDHit *ahit)
              : S4VExactHit(0,                          // hit ID
                            0,                          // track ID
                            0,                          // mother track ID
                            ahit->GetPDGEncoding(),     // PDG code
                            0,                          // charge
                            TVector3(ahit->GetPrePosition().x()/cm, // pre pos
                                     ahit->GetPrePosition().y()/cm,
                                     ahit->GetPrePosition().z()/cm),
                            ahit->GetTof()/nanosecond,  // TOF
                            TVector3(0.,0.,0.),         // momentum
                            0,                          // energy
                            ahit->GetEnergyDeposit()/GeV, // energy deposit
                            0),                         // address ptr 
                fPreHitID(ahit->GetPreHitID()),         // prehit ID
                fBlockID(ahit->GetBlockID()),           // block ID
                fLayerID(ahit->GetLayerID()),           // layer ID
                fIsBarrel(ahit->GetIsBarrel()),            // is Barrel?
                fIsFront(ahit->GetIsFront()),              // is Front?
                fPreHitPtr(0)
{
}

//====================================================================
#include "J4MUDPreHit.hh"
#include "G4ParticleDefinition.hh"
#include "S4MUDPreHit.h"

//---------------------------------------------------------------------
S4MUDPreHit::S4MUDPreHit(J4MUDPreHit *p)
            : S4VExactHit(p->GetPreHitID(),                       // prehit ID
                          p->GetTrackID(),                        // track ID
                          0,                                      // mother track ID
                          p->GetPDGEncoding(),                    // PDG code
                          p->GetCharge(),                         // charge
                          TVector3(p->GetPrePosition().x()/cm, // position
                                   p->GetPrePosition().y()/cm,
                                   p->GetPrePosition().z()/cm),
                          p->GetTof()/nanosecond,                 // TOF
                          TVector3(p->GetMomentum().x()/GeV,      // momentum
                                   p->GetMomentum().y()/GeV,
                                   p->GetMomentum().z()/GeV),
#if 0
                          p->GetKineticEnergy()/GeV,              // energy
#else
                          p->GetTotalEnergy()/GeV,                // energy
#endif
                          0,                                      // edep
                          0),                                     // address ptr
              fParticleName(p->GetParticle()->GetParticleName()), // particle name
              fPostHitID(p->GetCALPostHitID())                       // post hit ID
{
}

//====================================================================
#include "J4BreakPoint.hh"
#include "S4BreakPoint.h"

S4BreakPoint::S4BreakPoint(const J4BreakPoint *j4bpp)
             :fTrackID(j4bpp->GetTrackID()),
	      fParentID(j4bpp->GetParentID()),
	      fCharge(j4bpp->GetCharge()),
	      fParticleID(j4bpp->GetParticleID()),
	      fGlobalTime(j4bpp->GetGlobalTime()/ns),
	      fPosition(j4bpp->GetPosition().x()/cm,
                        j4bpp->GetPosition().y()/cm,
                        j4bpp->GetPosition().z()/cm),
	      fFourMomentum(j4bpp->GetFourMomentum().x()/GeV,
                            j4bpp->GetFourMomentum().y()/GeV,
                            j4bpp->GetFourMomentum().z()/GeV,
                            j4bpp->GetFourMomentum().t()/GeV),
	      fKineticEnergy(j4bpp->GetKineticEnergy()/GeV),
	      fPFOPtr(0)
{
  fgBreakPointMap.insert(std::pair<Int_t, S4BreakPoint*>(fTrackID, this));
}


//====================================================================
#include "J4OptDetHit.hh"
#include "S4OptDetExactHit.h"
S4OptDetExactHit::S4OptDetExactHit(const J4OptDetHit *hitp)
            : S4VExactHit(hitp->GetHitNumber(),           // hit ID
                          hitp->GetTrackID(),             // track ID
                          hitp->GetMotherTrackID(),       // mother track ID
                          hitp->GetPDGEncoding(),         // particle ID
                          (Int_t)hitp->GetCharge(),       // charge
                          (1.0/cm)*TVector3(hitp->GetPrePosition().x(),
				   hitp->GetPrePosition().y(),
				   hitp->GetPrePosition().z()),
                          hitp->GetTof(),                 // TOF
                          (1.0/GeV)*TVector3(hitp->GetMomentum().x(),
				   hitp->GetMomentum().y(),
				   hitp->GetMomentum().z()),
                          hitp->GetTotalEnergy()   / GeV, // total energy
                          hitp->GetEnergyDeposit() / GeV, // energy deposit
                          0,                              // address ptr
                          0),                             // event ID 
	      fDetID(((J4OptDetHit*)hitp)->GetDetID()),
	      fDetSerialID(((J4OptDetHit*)hitp)->GetDetSerialID()),
	      fPostPosition(((J4OptDetHit*)hitp)->GetPostPosition().x()/cm,
			    ((J4OptDetHit*)hitp)->GetPostPosition().y()/cm,
			    ((J4OptDetHit*)hitp)->GetPostPosition().z()/cm)
{
}



#endif

