//*************************************************************************
//* ----------------
//* S4BCALMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*	S4VMCTruthBuf:
//* (Provides)
//*	S4BCALMCTruthBuf		: MCTruth data buffer
//* (Update Record)
//*	2004/11/24  S.Yamamoto	Original version.
//*************************************************************************

#include <sstream>
#include <iomanip>
#include <iostream>
#include "S4TPCMCTruthBuf.h"
#include "S4BCALMCTruthBuf.h"
#include "S4BCALExactHit.h"
// #include "S4BCALPreHit.h"

#include "JSFSteer.h"

#define __MMTOCM__   0.100
#define __MEVTOGEV__ 0.001

using namespace std;

ClassImp(S4BCALMCTruthBuf)


//===========================================================
// Constructor -----------------------------------------------

S4BCALMCTruthBuf::S4BCALMCTruthBuf(const Char_t *name,
                                 const Char_t *title, 
                                 S4BCALMCTruth *module,
                                 Int_t         bufsize) 
               : S4VMCTruthBuf(name, title, module, bufsize),
                 fInputBufPtr(0),
                 fPreHitsPtr(0)
{
}

S4BCALMCTruthBuf::S4BCALMCTruthBuf(S4BCALMCTruth *module,
                                 const Char_t *name, 
                                 const Char_t *title,
                                 Int_t         bufsize) 
               : S4VMCTruthBuf(name, title, module, bufsize),
                 fInputBufPtr(0),
                 fPreHitsPtr(0)
{
   cerr << "constructor for S4BCALMCTruthBuf is called..." << endl;
}

//===========================================================
// Destructor -----------------------------------------------

S4BCALMCTruthBuf::~S4BCALMCTruthBuf()
{
}

//===========================================================
// ReadFile -------------------------------------------------
 
Bool_t S4BCALMCTruthBuf::ReadFile()
{
  /*
  cerr << " S4BCALMCTruthBuf::ReadFile() is called..." << endl;

  // Set Ownership Flag -------//
  GetExactHitsPtr()->SetOwner();
  if (!GetPreHitsPtr()) fPreHitsPtr = new TObjArray;
  GetPreHitsPtr()->SetOwner(); 

  ifstream & in = *S4VMCTruth::GetInputStreamPtr();
  // Seek CAL header -------//

  Int_t eventno = Seek("*****_EXPName:CAL_output_start_*****");
  if (eventno < 0)  return kTRUE;  // -ve means faiure to find one


  // read data -------//
  Char_t   dummy[1024];
  Int_t    preHitID, cellID, pcode, isEM, isBarrel;
  Double_t cellx, celly, cellz, posx, posy, posz; 
  Double_t edep, tof;
  TVector3 pos;
  Int_t    postHitID, trackID, motherTrackID;
  Double_t etot, mx, my, mz, charge;
  TVector3 mom;
  	
  
  Int_t hitID = 0;    // very temporary!!!!

  
  while (!in.eof()) {
    in >> dummy;
    if (! strncmp(dummy,"*****_EXPName:CAL_output_end_*****" ,34) ) break;
    if (!strncmp(dummy,"calhit", 6)) { 
      in >> preHitID     >> pcode 
	 >> cellID  >> isEM  >> isBarrel 
	 >> edep  >> tof  
         >> posx  >> posy  >> posz 
         >> cellx >> celly >> cellz
	 >> trackID >> motherTrackID; 

      posx      *= __MEVTOGEV__ * __MMTOCM__;
      posy      *= __MEVTOGEV__ * __MMTOCM__;
      posz      *= __MEVTOGEV__ * __MMTOCM__;

      cellx     *= __MMTOCM__;
      celly     *= __MMTOCM__;
      cellz     *= __MMTOCM__;
      edep      *= __MEVTOGEV__;

      pos.SetX(posx/edep);
      pos.SetY(posy/edep);
      pos.SetZ(posz/edep);

   
      S4BCALExactHit *datum 
	= new S4BCALExactHit(hitID,                 // hit ID
			    pcode,                 // PDG code
			    pos,                   // position [cm]
                            tof,                   // TOF      [nsec]
                            edep,                  // energy deposit [GeV]
			    preHitID,              // pre hit ID
			    cellID,                // cell ID
                            isBarrel,              // energy deposit [GeV]
                            isEM,                  // energy deposit [GeV]
                            TVector3(cellx, celly, cellz), // cell position [cm]
                            TVector3(posx, posy, posz),
			    trackID,motherTrackID);   // E-weighted X

      GetExactHitsPtr()->Add(datum);	
      hitID++;  // very temporary!!!!

    } else {    // PretHit
      in >> postHitID >> preHitID 
         >> trackID   >> motherTrackID  
         >> etot 
	 >> mx        >> my      >> mz  
	 >> pcode     >> charge  >> tof
         >> posx      >> posy    >> posz ;
            
      pos.SetX(posx * __MMTOCM__);
      pos.SetY(posy * __MMTOCM__);
      pos.SetZ(posz * __MMTOCM__);

      mom.SetX(mx * __MEVTOGEV__);
      mom.SetY(my * __MEVTOGEV__);
      mom.SetZ(mz * __MEVTOGEV__);

      etot      *= __MEVTOGEV__;
 
      S4BCALPreHit *datum 
	= new S4BCALPreHit(postHitID,             // posthit ID
			  preHitID,              // prehit ID
			  trackID,               // track ID 
			  motherTrackID,         // mother track ID
			  pcode,                 // PDG code
			  charge,                // charge
			  pos,                   // Injection position [cm]
			  tof,                   // TOF      [nsec]
			  mom,                   // momentum [GeV] 
			  etot,                  // energy
			  edep,                  // energy deposit [GeV] 
			  0);                    // address ptr
	
      GetPreHitsPtr()->Add(datum);	
    }
  }
  */
  return kTRUE;
}

//===========================================================
// UpdatePtrs -----------------------------------------------

void S4BCALMCTruthBuf::UpdatePtrs()
{
  /*
  TObjArray *ehitsp   = GetExactHitsPtr();  // exact hists collection
  TObjArray *prehitsp = GetPreHitsPtr();    // prehits collection

  TObjArray *poshitsp = static_cast<S4TPCMCTruthBuf *>(
			   gJSF->FindModule("S4TPCMCTruth")->EventBuf()
		        )->GetPostHitsPtr(); // post hit collection

  TIter next(ehitsp);
  S4BCALExactHit *ehitp = 0;
  while ((ehitp = static_cast<S4BCALExactHit *>(next()))) {
    Int_t prehitid = ehitp->GetPreHitID();
    S4BCALPreHit  *prehitp = prehitid == -1 
                           ? 0
                           : static_cast<S4BCALPreHit *>(prehitsp->At(prehitid));
    ehitp->SetPreHitPtr(prehitp);
    Int_t poshitid = prehitp->GetPostHitID();
    S4TPCPostHit *poshitp = poshitid == -1 
                          ? 0
                          : static_cast<S4TPCPostHit *>(poshitsp->At(poshitid));
    prehitp->SetPostHitPtr(poshitp);
  }
  */
}
