///////////////////////////////////////////////////////////////////
//
//  CT Hit class in ROOT
//
//$Id: S4CTExactHit.cxx,v 1.2 2006/07/03 07:14:23 miyamoto Exp $
//
//////////////////////////////////////////////////////////////////

#include <iostream>
#ifndef __HasJSFFromLCIO__
#include "G4Tubs.hh"
#include "J4CTLayerHit.hh"
#endif
#include "S4CTExactHit.h"
using namespace std;

ClassImp(S4CTExactHit)

// ------------------------------------------------------------------
S4CTExactHit::S4CTExactHit():
  fLayerNo(0), fWireNo(0), fCellID(0), fTrackID(0), fUniqueHitID(0),
  fMotherTrackID(0), fPDGParticleID(0), 
  fDeltaPhiAngle(0.0), fStartPhiAngle(0.0),
  fDriftLength(0.0),
  fPrePosition(0.0, 0.0, 0.0), fPostPosition(0.0, 0.0, 0.0),
  fHitPosition(0.0, 0.0, 0.0), fMomentum(0.0, 0.0, 0.0), 
  fEnergyDeposit(0.0), fTof(0.0)
{
}
#ifndef __HasJSFFromLCIO__
// ------------------------------------------------------------------
S4CTExactHit::S4CTExactHit(J4CTLayerHit *ahit)
{

  J4VComponent *vc=ahit->GetComponent();

  fWireNo=vc->GetMyID();
  fLayerNo=vc->GetMother()->GetMother()->GetMyID();
  fTrackID=ahit->GetTrackID();

  fPDGParticleID = ahit->GetParticle()->GetPDGEncoding();   
  

  fDeltaPhiAngle = ((G4Tubs*)vc->GetSolid())->GetDeltaPhiAngle();
  fStartPhiAngle = ((G4Tubs*)vc->GetSolid())->GetStartPhiAngle();

  G4ThreeVector pos  = ahit->GetPrePosition();

  Double_t ghphi    = pos.phi();
  while (ghphi < 0)      ghphi += 2*TMath::Pi();
  while (ghphi > 2*M_PI) ghphi -= 2*TMath::Pi();   
  Double_t cellphi = ghphi - fStartPhiAngle;
  while(cellphi < 0)      cellphi += 2*TMath::Pi();
  while(cellphi > 2*TMath::Pi()) cellphi -= 2*TMath::Pi();   
  fCellID  = (Int_t)(cellphi/fDeltaPhiAngle);
		
  fPrePosition=TVector3( ahit->GetPrePosition().x()/cm, ahit->GetPrePosition().y()/cm,
			 ahit->GetPrePosition().z()/cm);
  fPostPosition=TVector3( ahit->GetPostPosition().x()/cm, ahit->GetPostPosition().y()/cm,
			  ahit->GetPostPosition().z()/cm);
  fHitPosition=TVector3( pos.x()/cm, pos.y()/cm, 
			  pos.z()/cm);
  fMomentum=TVector3( ahit->GetMomentum().x()/GeV, ahit->GetMomentum().y()/GeV,
		      ahit->GetMomentum().z()/GeV);
  fEnergyDeposit=ahit->GetEnergyDeposit();
  fTof=ahit->GetTof();	


}
#endif

// ------------------------------------------------------------------
S4CTExactHit::~S4CTExactHit()
{
}

// ------------------------------------------------------------------
void S4CTExactHit::Print()
{
  std::cout << " Layer no=" << GetLayerNo();
  std::cout << " Wire no=" << GetWireNo();
  std::cout << " Cell ID=" << GetCellID();
  std::cout << " Track ID=" << GetTrackID();
  std::cout << " Unique Hit ID=" << GetUniqueHitID();
  std::cout << " Mother Track ID=" << GetMotherTrackID();
  std::cout << " PDG Track ID=" << GetPDGParticleID()  << std::endl;

  std::cout << " DeltaPhiAngle=" << GetDeltaPhiAngle() ;
  std::cout  << " StartPhiAngle=" << GetStartPhiAngle() ;
  std::cout  << " DriftLength=" << GetDriftLength() ;
  std::cout  << " EnergyDeposit=" << GetEnergyDeposit() ;
  std::cout  << " Tof=" << GetTof() ;
  std::cout  << std::endl;

  std::cout << "PrePosition(" << GetPrePosition().x() ;
  std::cout  << "," << GetPrePosition().y() << "," << GetPrePosition().z() << ")" ;
  std::cout  << "PostPosition(" << GetPostPosition().x() ;
  std::cout  << "," << GetPostPosition().y() << "," << GetPostPosition().z() << ")" ;
  std::cout  << std::endl;
  //  cout << "HitPosition(" << GetHitPosition().x() ;
  //  cout << "," << GetHitPosition().y() << "," << GetHitPosition().z() << ")" ;
  std::cout << "Momentum(" << GetMomentum().x() ;
  std::cout  << "," << GetMomentum().y() << "," << GetMomentum().z() << ")" ;
  std::cout    << std::endl;
  

}
