//*************************************************************************
//* ------------------
//* S4GENMCTruth Class
//* ------------------
//* (Description)
//*     A class to book keep generator information.
//* (Requires)
//* (Provides)
//*	S4GENMCTruthBuf		: data buffer
//*	S4GENMCTruth		: steering class
//* (Update Record)
//*	2005/04/01  K.Fujii	Original version.
//*************************************************************************

#include <iostream>
#include "TLorentzVector.h"
#include "S4GENMCTruth.h"
#include "S4GENMCTruthBuf.h"
#include "S4PrimaryVertex.h"

using namespace std;

ClassImp(S4GENMCTruth)

//=====================================================================
//* constructor -------------------------------------------------------

S4GENMCTruth::S4GENMCTruth(const Char_t *fname, 
                           const Char_t *name,
                           const Char_t *title)
           : S4VMCTruth(name, title),
             fInputFileName((Char_t *)fname)

{
#ifdef __DEBUG__
   cerr << " constructor for S4GENMCTruth is called..." << endl;
#endif
}

//=====================================================================
//* destructor --------------------------------------------------------

S4GENMCTruth::~S4GENMCTruth()
{
#ifdef __DEBUG__
   cerr << " S4GENMCTruth will be deleted...fEventBuf is " 
        << (Int_t)fEventBuf << endl;
#endif
   if(fEventBuf) delete fEventBuf;
   if(!fInputStream) fInputStream->close();
}

//=====================================================================
//* Initialize  -------------------------------------------------------

Bool_t S4GENMCTruth::Initialize()
{
#ifdef __DEBUG__
   cerr << "S4GENMCTruth::Initialize() is called..." << endl;
#endif
   if (!fEventBuf) fEventBuf = new S4GENMCTruthBuf(this);

   return S4VMCTruth::Initialize();
}

//=====================================================================
//* BeginRun  ---------------------------------------------------------
                                                                                
Bool_t S4GENMCTruth::BeginRun(Int_t nrun)
{
#ifdef __DEBUG__
   cerr << "S4GENMCTruth::BeginRun() is called..." << endl;
#endif
    return S4VMCTruth::BeginRun(nrun);
}

//=====================================================================
//* Process   --------------------------------------------------------

Bool_t S4GENMCTruth::Process(Int_t ev)
{
   S4VMCTruth::Process(ev);
#ifdef __DEBUG__
   cerr << "S4GENMCTruth::Process() is called..." << endl;  
#endif
   S4GENMCTruthBuf *bufp = static_cast<S4GENMCTruthBuf *>(fEventBuf);

   if ( fFile->IsWritable() ) {
      if (bufp->GetInputBufPtr()) {
         bufp->GetExactHitsPtr()->Clear();
         TObjArray *ehitbufp   = bufp->GetExactHitsPtr(); 
         TIter next(bufp->GetInputBufPtr());
         TObject *objp = 0;
         while ((objp = next())) ehitbufp->Add(objp);
#ifdef __DEBUG__
         cerr << "ITHitsPtr = " << (void *)bufp->GetInputBufPtr() << endl;
         cerr << "Nhits     = " << bufp->GetInputBufPtr()->GetEntries() << endl;
#endif
      } else {
         bufp->GetExactHitsPtr()->Delete();
         bufp->ReadFile();
      }
#ifdef __DEBUG__
      cerr << "S4GENMCTruth::Process nhits =" 
           << bufp->GetExactHitsPtr()->GetEntries() << endl;
#endif
#ifdef __DEBUG__
      DebugPrint();
#endif
      return kTRUE;
   } else {
      cerr << "S4GENMCTruth::Process ExacthitsBuffer is not updated. " << endl;
      return kTRUE;
   }
}


//=====================================================================
//* DebugPrint -------------------------------------------------------

void S4GENMCTruth::DebugPrint(const Char_t *opt)
{
   cerr << " ---- S4GENMCTruth::DebugPrint ------------- " << endl;

   S4GENMCTruthBuf *bufp = (S4GENMCTruthBuf *)fEventBuf;
   TIter next(bufp->GetInputBufPtr()); 
   S4PrimaryVertex *pvp;
   while ((pvp = static_cast<S4PrimaryVertex *>(next()))) {
      pvp->Print(opt);
   }
   TLorentzVector pvcm = bufp->GetPVcm();
   cerr << " (E,Px,Py,Pz)_cm = "
        << " (" << pvcm.E()
        << ", " << pvcm.Px()
        << ", " << pvcm.Py()
        << ", " << pvcm.Pz() << ")" << endl;
}
