//*************************************************************************
//* ----------------
//* S4TPCMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*	S4VMCTruthBuf:
//* (Provides)
//*	S4TPCMCTruthBuf		: MCTruth data buffer
//* (Update Record)
//*	2001/08/20  A.Yamaguchi	Original version.
//*************************************************************************

#include <iomanip>
#include <iostream>
#include <fstream>
#include "S4TPCMCTruthBuf.h"
#include "S4TPCExactHit.h"
#include "S4TPCPostHit.h"

//#define __DEBUG__
using namespace std;

ClassImp(S4TPCMCTruthBuf)


//===========================================================
// Constructor -----------------------------------------------

S4TPCMCTruthBuf::S4TPCMCTruthBuf(const Char_t *name,
                                 const Char_t *title, 
                                 S4TPCMCTruth *module,
                                 Int_t         bufsize) 
               : S4VMCTruthBuf(name, title, module, bufsize),
                 fPostHitsPtr(0),
                 fInputBufPtr(0)
{
}

S4TPCMCTruthBuf::S4TPCMCTruthBuf(S4TPCMCTruth *module,
                                 const Char_t *name, 
                                 const Char_t *title,
                                 Int_t         bufsize) 
               : S4VMCTruthBuf(name, title, module, bufsize),
                 fPostHitsPtr(0),
                 fInputBufPtr(0)
{
}

//===========================================================
// Destructor -----------------------------------------------

S4TPCMCTruthBuf::~S4TPCMCTruthBuf()
{
}

//===========================================================
// ReadFile -----------------------------------------------

Bool_t S4TPCMCTruthBuf::ReadFile()
{
  // Set Ownership Flag -------//
  GetExactHitsPtr()->SetOwner();
  if (!GetPostHitsPtr()) fPostHitsPtr = new TObjArray;
  GetPostHitsPtr()->SetOwner();
 
  ifstream & in = *S4VMCTruth::GetInputStreamPtr();
  // Seek TPC header -------//
 

  Int_t eventno = Seek("*****_EXPName:TPC_output_start_*****");
  if (eventno < 0)  return kTRUE;  // -ve means faiure to find one


  // read data -------//
  Char_t   dummy[1024];
  Int_t    layerID, trackID, 
           mothertrkID, pcode, charge;
  Double_t prex, prey, prez, postx, posty, postz; 
  Double_t mx, my, mz, totE, edep, tof;
  TVector3 pre, post, hit, momentum;
  	
  
  Int_t hitID = 0;    // very temporary!!!!

   static const Double_t kMMToCM   = 0.1;
   static const Double_t kMevToGeV = 0.001;
  
  while (!in.eof()) {
    in >> dummy;
    if (! strncmp(dummy,"*****_EXPName:TPC_output_end_*****" ,34) ) break;
    if (!strncmp(dummy,"Layer", 5)) { 
      in >> layerID     >> trackID 
         >> mothertrkID >> pcode >> charge  
         >> prex  >> prey  >> prez 
         >> postx >> posty >> postz 
         >> mx    >> my    >> mz 
         >> totE  >> edep  >> tof;
    
      prex  *= kMMToCM;
      prey  *= kMMToCM;
      prez  *= kMMToCM;
      postx *= kMMToCM;
      posty *= kMMToCM;
      postz *= kMMToCM;
      mx    *= kMevToGeV;
      my    *= kMevToGeV;
      mz    *= kMevToGeV;
      totE  *= kMevToGeV;
      edep  *= kMevToGeV;
   
      pre.SetX(prex);
      pre.SetY(prey);
      pre.SetZ(prez);
    
      post.SetX(postx);
      post.SetY(posty);
      post.SetZ(postz);
    
      momentum.SetX(mx);
      momentum.SetY(my);
      momentum.SetZ(mz);

 
      S4TPCExactHit *datum 
            = new S4TPCExactHit(eventno,      layerID, trackID,
                                mothertrkID, hitID,   pcode, 
                                charge,       pre,         post,
                                momentum,     totE,        edep,        tof);
  
      GetExactHitsPtr()->Add(datum);	
      hitID++;  // very temporary!!!!

    } else {    // PostHit
      in >> hitID >> trackID 
         >> mothertrkID    >> pcode >> charge  
         >> postx >> posty >> postz 
         >> mx    >> my    >> mz 
         >> totE  >> tof;
    
      post.SetX(postx * kMMToCM);
      post.SetY(posty * kMMToCM);
      post.SetZ(postz * kMMToCM);
    
      momentum.SetX(mx * kMevToGeV);
      momentum.SetY(my * kMevToGeV);
      momentum.SetZ(mz * kMevToGeV);

      totE      *= kMevToGeV;
 
      S4TPCPostHit *datum 
            = new S4TPCPostHit(eventno, trackID, mothertrkID, hitID, pcode,
                               charge,  post,    momentum,    totE,  tof);
  
      GetPostHitsPtr()->Add(datum);	
    }
  }

  return kTRUE;
}

//===========================================================
// MergeHits  -----------------------------------------------

void S4TPCMCTruthBuf::MergeHits()
{
#ifdef __DEBUG__
	cerr << "S4TPCMCTruthBuf::MergeHits() called >>>>>>>>>" << endl;
#endif
   S4TPCExactHit::SetSortBy(S4VExactHit::kTrackID);
   TObjArray *hitbufp = GetExactHitsPtr();
   if (!hitbufp->GetEntries()) return;
   hitbufp->UnSort();
   hitbufp->Sort(); 
   TObjArray track(hitbufp->GetEntries());

   TIter next(hitbufp); 
   S4TPCExactHit *ehitp = static_cast<S4TPCExactHit *>(next());
   Int_t last = ehitp->GetTrackID(); 
   next.Reset();
   while ((ehitp = static_cast<S4TPCExactHit *>(next()))) {
      if (ehitp->GetTrackID() == last) {
         track.Add(ehitp);
      } else {
         last = ehitp->GetTrackID();
         CleanUp(track);
         track.Clear();
      }         
   }
   if (track.GetEntries()) CleanUp(track);
   hitbufp->Compress();
}

//===========================================================
// CleanUp  -------------------------------------------------

void S4TPCMCTruthBuf::CleanUp(TObjArray &track)
{
#ifdef __DEBUG__
	cerr << "S4TPCMCTruthBuf::CleanUp() called >>>>>>>>>" << endl;
#endif
   S4TPCExactHit::SetSortBy(S4VExactHit::kTof);
   track.UnSort(); 
   track.Sort(); 
   TIter nexthit(&track);
   Int_t nhits = track.GetEntries();
   for (Int_t i = 1; i < nhits; i++) { 
      S4TPCExactHit *lastp = static_cast<S4TPCExactHit *>(track[i-1]);
      S4TPCExactHit *curp  = static_cast<S4TPCExactHit *>(track[i]);

#ifdef __DEBUG__
      //curp->DebugPrint();
      Int_t padnolast = lastp->GetPadRowNumber();
      Int_t padnocur = curp->GetPadRowNumber();
      if (padnolast == padnocur) {
         cerr << "---------" << endl;
         cerr << " padrowlast = " << padnolast;
         cerr << " Xpos = ("  
              << lastp->GetPostPosition().X() << ", " 
              << lastp->GetPostPosition().Y() << ", " 
              << lastp->GetPostPosition().Z()
              << ")" << endl;
         cerr << " padrowcur  = " << padnocur;
         cerr << " Xpre = ("  
              << curp->GetPrePosition().X() << ", " 
              << curp->GetPrePosition().Y() << ", " 
              << curp->GetPrePosition().Z()
              << ")" << endl;
      }
#endif
#if __G4VERSION__ < 81
      if ((lastp->GetPadRowNumber() == curp->GetPadRowNumber()) &&
          (lastp->GetPostPosition() == curp->GetPrePosition())) {
#else
      static const Double_t kCurTolerance = 1.e-4;
#if 0
      if ((lastp->GetPadRowNumber() == curp->GetPadRowNumber()) &&
          ((lastp->GetPostPosition()-curp->GetPrePosition()).Mag() < kCurTolerance)) {
#else
#if 0
      TVector3 dx = curp->GetPrePosition() - lastp->GetPostPosition();
      if ((lastp->GetPadRowNumber() == curp->GetPadRowNumber()) &&
          (TMath::Abs(dx.X()) < kCurTolerance &&
	   TMath::Abs(dx.Y()) < kCurTolerance &&
	   TMath::Abs(dx.Z()) < kCurTolerance)) {
#else
      if (lastp->GetPadRowNumber() == curp->GetPadRowNumber()) {
#endif
#endif
#endif
         curp->SetPrePosition(lastp->GetPrePosition());
         curp->SetEnergyDeposit(lastp->GetEnergyDeposit()
                              + curp->GetEnergyDeposit());
         if (curp->GetPrePosition().Pt() < curp->GetPostPosition().Pt()) {
            curp->SetPosition(lastp->GetPosition());
            curp->SetPadRowRadius(lastp->GetPadRowRadius());
            curp->SetPhi(lastp->GetPhi());
            curp->SetZdrift(lastp->GetZdrift());
            curp->SetSide(lastp->GetSide());
            curp->SetTOF(lastp->GetTOF());
            curp->SetMomentum(lastp->GetMomentum());
            curp->SetTotalEnergy(lastp->GetTotalEnergy());
         }
         GetExactHitsPtr()->Remove(lastp);
         if (!fInputBufPtr) delete lastp;
#ifdef __DEBUG__
	 cerr << " Removed lastp = " << lastp << endl;
	 cerr << " --- " << lastp << endl;
#endif
      }
   }
}
