//*************************************************************************
//* ----------------
//* S4VTXMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*	TObject:
//*	TClonesArray:
//*	VTXUniversalRecordHeader:
//*	VTXRawADCDatum:
//* (Provides)
//*	S4VTXMCTruthBuf		: raw data buffer
//*	S4VTXMCTruth		: steering class
//* (Update Record)
//*	2004/06/02  A.Yamaguchi	Original version.
//*************************************************************************

#include <iostream>
#include "S4VTXMCTruth.h"
#include "S4VTXExactHit.h"
#include "S4VTXMCTruthBuf.h"
using namespace std;

//#define __DEBUG__

ClassImp(S4VTXMCTruth)

//=====================================================================
//* constructor -------------------------------------------------------

S4VTXMCTruth::S4VTXMCTruth(const Char_t *fname, 
                           const Char_t *name,
                           const Char_t *title)
            : S4VMCTruth(name, title),
              fInputFileName((Char_t *)fname)
{
#ifdef __DEBUG__
   cerr << " constructor for S4VTXMCTruth is called..." << endl;
#endif
   fEventBuf = new S4VTXMCTruthBuf(this);
}

//=====================================================================
//* destructor --------------------------------------------------------

S4VTXMCTruth::~S4VTXMCTruth()
{
#ifdef __DEBUG__
   cerr << " S4VTXMCTruth will be deleted...fEventBuf is " 
        << (Int_t)fEventBuf << endl;
#endif
   if(fEventBuf) delete fEventBuf;
   if(!fInputStream) fInputStream->close();
}

//=====================================================================
//* BeginRun  --------------------------------------------------------

Bool_t S4VTXMCTruth::BeginRun(Int_t nrun)
{
#ifdef __DEBUG__
   cerr << "S4VTXMCTruth::BeginRun() is called..." << endl;
#endif
    return S4VMCTruth::BeginRun(nrun);

}

//=====================================================================
//* Process   --------------------------------------------------------

Bool_t S4VTXMCTruth::Process(Int_t ev)
{
   S4VMCTruth::Process(ev);
#ifdef __DEBUG__
   cerr << "S4VTXMCTruth::Process() is called..." << endl;  
#endif
   S4VTXMCTruthBuf *bufp = (S4VTXMCTruthBuf *)fEventBuf;

   if ( fFile->IsWritable() ) {
      if (bufp->GetInputBufPtr()) {
         bufp->GetExactHitsPtr()->Clear();
         TObjArray *ehitbufp   = bufp->GetExactHitsPtr(); 
         TIter next(bufp->GetInputBufPtr());
         TObject *objp = 0;
         while ((objp = next())) {ehitbufp->Add(objp);}
#ifdef __DEBUG__
         cerr << " VTXHitsPtr = " << (void *)bufp->GetInputBufPtr() << endl;
         cerr << " Nhits      = " << bufp->GetInputBufPtr()->GetEntries() << endl;
#endif
        
      } else {
         bufp->GetExactHitsPtr()->Delete();
         bufp->ReadFile();
      }
#ifdef __DEBUG__
      cerr << "S4VTXMCTruth::Process nhits =" 
           << bufp->GetExactHitsPtr()->GetEntries() << endl;
#endif
      bufp->MergeHits();
#ifdef __DEBUG__
      DebugPrint();
#endif
      return kTRUE;
   }
   else {
      cerr << "S4VTXMCTruth::Process ExacthitsBuffer is not updated. " << endl;
      return kTRUE;
   }
}


//=====================================================================
//* DebugPrint -------------------------------------------------------

void S4VTXMCTruth::DebugPrint(const Char_t *)
{
   S4VTXMCTruthBuf *bufp = (S4VTXMCTruthBuf *)fEventBuf;
   TIter next(bufp->GetExactHitsPtr()); 
   S4VTXExactHit *ehitp;
   while ((ehitp = static_cast<S4VTXExactHit *>(next()))) {
      ehitp->Print();
   }

}
