#ifndef __S4BabyHITWire__
#define __S4BabyHITWire__

//*************************************************************************
//* ---------------------
//* S4BabyHitWire Class
//* ---------------------
//* (Description)
//*	Class to store "S4BabyHit"s for a single wire.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4BabyHitWire		: Array containing "S4BabyHit"s
//* (Update Record)
//*	2002/08/28  R.Kuboshima
//*************************************************************************

class U4CDCHitCell;
class S4BabyHit;

#include "U4CDCHitWire.h"

#define __TMP__

class S4BabyHit;

class S4BabyHitWire : public U4CDCHitWire {

public:

  S4BabyHitWire(U4CDCTimeToXRelation *calib = 0);
  virtual ~S4BabyHitWire(); 
  
  virtual Int_t		Append(S4BabyHit *hit);
  virtual void 		DebugPrint(const Char_t *opt = "Brief");
  
  virtual void 		KillHits ();
  inline virtual void 	Revive() 	 { fKilled = kFALSE; }
  inline virtual Bool_t	IsKilled() const { return fKilled; }
  
  static void 		SetTwoHitSeparation(Double_t sep) 
  				{ fgTwoHitSeparation = sep; }  
  static void 		SetTwoHitSeparationZ(Double_t sepz) 
  				{ fgTwoHitSeparationZ = sepz; } 
  static void 		TPCMode(Bool_t mode = kTRUE) { fgTPCMode = mode; }
  				

public:
   enum {
      kEBelowThreshold     = 0x00010000,    // Two small pulse height.
      kECannotSeparate     = 0x00020000,    // Hit is killed becase of 2-hit dist.
      kEDeadChannel        = 0x00040000,    // Dead channel.
      kENoisyChannel       = 0x00080000,    // Noisy channel.
      kEModifiedByDeltaRay = 0x00100000     // Hit is modified because of DeltaRay.
   };
   
private:
  static void ChannelToWire(S4BabyHit *hit,
  			    Int_t &l, Int_t &c, Int_t &w, 
  			    Double_t &x, Double_t &y);
private:
  static Double_t 	fgTwoHitSeparation;	//
  static Double_t 	fgTwoHitSeparationZ;	//
  static Bool_t 	fgTPCMode;		//
  static Double_t       f2Tkpar[2];             //
  			    
  Bool_t fKilled;		// flag to tell hits are killed or not.

  ClassDef(S4BabyHitWire, 1) 	// Baby Hit Wire
};

#endif

