#ifndef __S4BabyTRACK__
#define __S4BabyTRACK__

//*************************************************************************
//* ---------------------
//* S4BabyTrack Class
//* ---------------------
//* (Description)
//*	Class to store "S4BabyHitCell"s for a single layer.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4BabyTrack		: Array containing "S4BabyHitCell"s
//* (Update Record)
//*	2002/08/28  R.Kuboshima
//*************************************************************************

#include "TObjArray.h"
#include "S4BabyVectorHit.h"
#include "JSFHelicalTrack.h"
#include "S4BabyTrackHit.h"

class S4BabyVectorHit;

class S4BabyTrack : public TObjArray {
public:
  S4BabyTrack();
  virtual ~S4BabyTrack();

  inline virtual Int_t	GetTrackID() const { return fTrackNumber; }
  
  inline virtual S4BabyTrack *GetMotherTrackPtr() const
			{ return fMotherTrackPtr; }
  inline virtual Int_t GetPDGCode() const;

  inline virtual void SetMotherTrackPtr(S4BabyTrack *trackp)
			{ fMotherTrackPtr = trackp; }

  virtual Int_t		Append(S4BabyVectorHit *vectorhit);
  virtual Bool_t	IsSortable() const;
  virtual Int_t		Compare(const TObject *obj) const;
  virtual void 		DebugPrint(const Char_t *opt = "Brief");
  
private:
  Bool_t 	   fDone		;	//
  Int_t		   fTrackNumber		;	// Track Number.
  S4BabyTrack     *fMotherTrackPtr      ;       //
 
  ClassDef(S4BabyTrack, 1) 		// Baby Track
};

//-----------------------------------------------------------
//  inline function
//-----------------------------------------------------------

Int_t S4BabyTrack::GetPDGCode() const
{
   TObjArray* vectorhit = (TObjArray*)((TObjArray *)this)->operator[](0);
   TObject*  trackhit  = vectorhit->operator[](0);
   Int_t pdgcode = ((S4BabyTrackHit *)trackhit)->GetPDGCode();
   return pdgcode;
}


#endif




