//*************************************************************************
//* --------------------
//* S4CALClusterMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes CALClusters.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2004/09/04  S.Yamamoto	Original version.
//*************************************************************************

#include <iostream>
#include "S4CALSmallCluster.h"
#include "S4CALSmallClusterMakerBuf.h"
#include "S4CALSmallClusterUtils.h"
#include "S4CALHitMaker.h"
#include "S4CALHit.h"
#include "S4CALExactHit.h"
#include "S4CALPreHit.h"
#include "S4HYBTrackMaker.h"
#include "S4HYBTrackMakerBuf.h"
#include "S4CALAddress.h"
#include "S4CALAddress.h"
#include "JSFHelicalTrack.h"

#include <cmath>

//#define __ELIMIT__

#ifndef __ELIMIT__
//#define __NLIMIT__
#endif

// GLD
//#define __MERGE__

// LDC
#define __MERGELDC__

//#define __DEBUG__

ClassImp(S4CALSmallClusterMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------

S4CALSmallClusterMakerBuf::S4CALSmallClusterMakerBuf(S4CALSmallClusterMaker *module, 
						     const Char_t      *name, 
						     const Char_t      *title, 
						     Int_t bufsize)
  : JSFEventBuf(name, title, (JSFModule*)module),
    fDone(kFALSE),
    fInputDataBufPtr(0), fOutputBufPtr(0), fBufSize(bufsize)
{
}

//=====================================================================
//* destructor --------------------------------------------------------

S4CALSmallClusterMakerBuf::~S4CALSmallClusterMakerBuf()
{
    if(fOutputBufPtr){
      fOutputBufPtr->Delete();
      delete fOutputBufPtr;
    }
}

//===================================================================== 
//* GetInputBufPtr ----------------------------------------------------

JSFEventBuf* S4CALSmallClusterMakerBuf::GetInputDataBufPtr()
{
  return fInputDataBufPtr;
}

//=====================================================================
//* GetOutputBufPtr ------------------------------------------------------

TObjArray* S4CALSmallClusterMakerBuf::GetOutputBufPtr()
{
   return fOutputBufPtr;
}

//=====================================================================
//* GetEntries --------------------------------------------------------

Int_t S4CALSmallClusterMakerBuf::GetEntries()
{
  if (!fDone) MakeSmallClusters();
   return fOutputBufPtr->GetEntries();
}

//=====================================================================
//* SetInputDataBuf ---------------------------------------------------

void S4CALSmallClusterMakerBuf::SetInputDataBufPtr(JSFEventBuf *buf)
{
   fDone = kFALSE;
   fInputDataBufPtr = buf;
   if (!fOutputBufPtr) fOutputBufPtr = new TObjArray(fBufSize);
   if (fOutputBufPtr->GetEntries() > 0) fOutputBufPtr->Delete();
}
//=====================================================================
//* DebugPrint --------------------------------------------------------
 
void S4CALSmallClusterMakerBuf::DebugPrint(const Char_t *)
{
#if 0
   if (fOutputBufPtr && fOutputBufPtr->GetEntries()) {
      TIter next(fOutputBufPtr);
      S4CALSmallCluster *cp = 0;
      while ((cp = static_cast<S4CALSmallCluster *>(next()))) {
         cp->DebugPrint();
      }
   }
#endif
}

//=====================================================================
//* MakeSmallClusters -------------------------------------------------

void S4CALSmallClusterMakerBuf::MakeSmallClusters()
{
   if (fDone) return; // check if small clusters are already made

   //---------- parameters for ILFinder -------------
   const Double_t ILmomcut   = 1.; // momentum cut for ILFinder (GeV)
   S4CALSmallClusterUtils::fgILEMTubeR      = 6.;      // cm
   S4CALSmallClusterUtils::fgILHDTubeR      = 20.;
   S4CALSmallClusterUtils::fgILMIPCutEnergy = 700.e-6; // GeV
   //------------------------------------------------

#if 0
   static Int_t nemmc = gJSF->Env()->GetValue("J4CAL.EM.MiniCone.NClones",3);
   static Int_t nemmt = gJSF->Env()->GetValue("J4CAL.EM.MiniTower.NClones",3);
   static Int_t nhdmc = gJSF->Env()->GetValue("J4CAL.HD.MiniCone.NClones",1);
   static Int_t nhdmt = gJSF->Env()->GetValue("J4CAL.HD.MiniTower.NClones",1);
#else
   Int_t nemmc = S4CALAddress::GetNMiniCones( kTRUE );
   Int_t nemmt = S4CALAddress::GetNMiniTowers( kFALSE );
   Int_t nhdmc = S4CALAddress::GetNMiniCones( kTRUE );
   Int_t nhdmt = S4CALAddress::GetNMiniTowers( kFALSE );

   std::cout << nemmc << " " << nemmt << " " << nhdmc << " " << nhdmt
	     << std::endl;

#endif

   fDone = kTRUE;
   fOutputBufPtr->SetOwner();        // buffer owns small clusters 

   SMAP hitmap = HitMapMaker();
   if( hitmap.size() == 0 )  return;
   SMAP::iterator hitr;


   //------------------ Find IL --------------------
   std::vector<S4CALSmallCluster*> sclvecIL;

   std::map<Int_t,S4HYBTrack*> trkmap = MakeTrackMap();
   std::map<Int_t,S4HYBTrack*>::iterator titr;

   //----- sort by momentum ----------
   std::multimap<Double_t,S4HYBTrack*> mommap;
   std::multimap<Double_t,S4HYBTrack*>::reverse_iterator momitr;
   for(titr = trkmap.begin(); titr != trkmap.end(); titr++){

#if 0
    JSF3DV mom = titr->second->GetHelicalTrackPtr()->GetMomentum(0.);
    Double_t costheta = TMath::Abs(mom.Z())/mom.Abs();
    if(mom.Abs() < ILmomcut - costheta) continue;
#else
    JSF3DV mom = titr->second->GetHelicalTrackPtr()->GetMomentum(0.);
    if(mom.Abs() < ILmomcut)            continue;
#endif

     mommap.insert(std::pair<Double_t,S4HYBTrack*>(mom.Abs(),titr->second));
   }
   //---------------------------------

   for(momitr = mommap.rbegin(); momitr != mommap.rend(); momitr++){

     std::pair<TVector3,std::vector<S4CALHit*> > ilpair;
     ilpair = S4CALSmallClusterUtils::ILFinder(hitmap, momitr->second);

     std::vector<S4CALHit*> ilinvec = ilpair.second;

     if(ilinvec.empty()) continue; // IL is not found

     S4CALSmallCluster* scl = new S4CALSmallCluster();

     for(Int_t k = 0; k < (Int_t) ilinvec.size(); k++){
       scl->Add(ilinvec[k]);
       ilinvec[k]->Lock();
     }

     (scl->fIsMIP)      = kTRUE;
     (scl->fILPos)      = ilpair.first;
     (scl->fILTrackPtr) = momitr->second;
     scl->Update();
     sclvecIL.push_back(scl);

   }
   //-----------------------------------------------


   //----- energy threshold ------
   Double_t ecut    = 3.e-3;   // GeV
   Double_t estep   = 0.1e-3;  // GeV
   //-----------------------------

   //---- small clusters vector(firstcellID,scl) ----------
   std::vector<std::pair<Int_t,S4CALSmallCluster*> > sclvec;
   //------------------------------------------------------


   //---------------------------------------------------------
   //---------------- start small clustering -----------------

   while(ecut > -1.*estep){ // energy threshold loop

     Int_t        cellID = 0;
     S4CALHit *firstcell = 0;

     // if second is true, it is already used as a center cell
     std::map<Int_t, Bool_t> usedCell;

     Int_t vecnum = 0;
     Int_t vecsize = (Int_t) sclvec.size();

     while(1){  // small clusters loop

       //-------- check if hit is remaining --------
       Bool_t remaining = kFALSE;
       for(hitr = hitmap.begin(); hitr != hitmap.end(); hitr++){
	 if(hitr->second->IsLocked()) continue;
	 if(hitr->second->GetEnergyDeposit() > ecut){
	   remaining = kTRUE;
	   break;
	 }
       }
       if(!remaining) break; // reset ecut and try again
       //-------------------------------------------

       //------ prepare small cluster etc. ---------
       S4CALSmallCluster* scl;
       usedCell.clear();

       if(vecnum < vecsize){ // use existing small cluster
	 cellID    = (sclvec[vecnum]).first;
	 firstcell = hitmap.find(cellID)->second;
	 scl       = (sclvec[vecnum]).second;

	 std::vector<S4CALHit*> hitv = scl->GetCALHitsVector();
	 for(Int_t k = 0; k < (Int_t) hitv.size(); k++){
	   Int_t tmpID = hitv[k]->GetCellID();
	   usedCell[tmpID] = kFALSE;	   
	 }

       }else{ // make new small cluster
	 firstcell = FirstCellFinder(hitmap,ecut);
	 cellID    = firstcell->GetCellID();

	 scl = new S4CALSmallCluster();
	 scl -> Add(firstcell);
	 firstcell -> Lock();
	 sclvec.push_back(std::pair<Int_t,S4CALSmallCluster*>(cellID,scl));
	 usedCell[cellID] = kFALSE;
       }

       Double_t cut = ecut;
       //-------------------------------------------

#ifdef __ELIMIT__
       scl->Update();
       if( (scl->GetEnergy()) > scl->GetMaxEnergyCut() ){
	 vecnum++;
	 continue;
       }
#endif

#ifdef __NLIMIT__
       scl->Update();
       if( (scl->GetNHits()) > scl->GetMaxNHitsCut() ){
	 vecnum++;
	 continue;
       }
#endif

       Bool_t isEM       = firstcell->IsEM();
       Bool_t isBarrel   = (firstcell->GetCALType() & ( kgBaEm | kgBaHd ));
       TVector3 firstpos = firstcell->GetCellPosition();

       //-------------------------------------------
       // add neighbor cells into a small cluster
       //-------------------------------------------
       while ( cellID > 0 ) { // add hits into a small cluster

	 usedCell[cellID] = kTRUE;  // if true, it is already used as a center

	 //---------------------------------------
	 // add neighbor cells into small cluster
	 //---------------------------------------
         //* Find neighbor cells  

	 std::vector<Int_t> neighborVector = S4CALAddress::GetListOfNeighbors( cellID, isEM, isBarrel );
	 for ( Int_t i = 0; i < (Int_t)neighborVector.size(); i++ ) {
	   Int_t neighborcell = neighborVector[i];
	   if ( hitmap.count(neighborcell)<1 ) continue; // not fired
	   hitr = hitmap.find( neighborcell );
	   if( !(hitr->second->IsLocked()) && ((hitr->second->IsEM()) == isEM) && (hitr->second->GetEnergyDeposit() > cut) ){
	     hitr = hitmap.find(neighborcell);
             scl->Add( hitr->second );
             usedCell[neighborcell] = kFALSE;
             hitr->second->Lock();
	   }
	 }

	 //--- find next center ---------------

#ifdef __ELIMIT__
	 //--- check small cluster enrgy -----
	 scl->Update();
	 if( (scl->GetEnergy()) > scl->GetMaxEnergyCut() )
	   break;
	 //-----------------------------------    
	 Double_t tmpdist = 1.e+5;
#endif

#ifdef __NLIMIT__
	 //--- check small cluster enrgy -----
	 scl->Update();
	 if( (scl->GetNHits()) > scl->GetMaxNHitsCut() )
	   break;
	 //-----------------------------------    
	 Double_t tmpdist = 1.e+5;
#endif

	 cellID = 0;
	 std::map<Int_t, Bool_t>::iterator itr;

	 for(itr = usedCell.begin(); itr != usedCell.end(); itr++ ){
	   if( !(itr -> second) ){ // this is not checked yet
#ifdef __ELIMIT__
	     TVector3 tmppos = hitmap.find(itr->first)->second->GetCellPosition();
	     if( (firstpos-tmppos).Mag() < tmpdist ){ // choose the nearest one
	       cellID = itr -> first;
	       tmpdist = (firstpos - tmppos).Mag();
	     }
#endif

#ifdef __NLIMIT__
	     TVector3 tmppos = hitmap.find(itr->first)->second->GetCellPosition();
	     if( (firstpos-tmppos).Mag() < tmpdist ){ // choose the nearest one
	       cellID = itr -> first;
	       tmpdist = (firstpos - tmppos).Mag();
	     }
#else
	     cellID = itr -> first; // anything OK
	     break;
#endif
	   }
	 }
	 //------------------------------------


       } // end of while(cellID > 0) loop

       scl -> Update();

       vecnum++; // sclvec number

     } // end of while(1)

     ecut -= estep;

   } // end of while(ecut > -1.*estep)

   //-------------- end of small clustering ------------------
   //---------------------------------------------------------


   //==========================================================
   // Merge test
#ifdef __MERGE__


   std::vector<S4CALSmallCluster*> msclvec; // merged small clusters

#if 1
   //---- dX dT based merging ------

   std::vector<S4CALSmallCluster*> test;
   for(Int_t k = 0; k < (Int_t) sclvec.size(); k++){
     test.push_back(sclvec[k].second);
   }

   S4CALSmallClusterUtils::SetEMdX(10.);
   S4CALSmallClusterUtils::SetHDdX(20.);
   S4CALSmallClusterUtils::SetEMdT(20.);
   S4CALSmallClusterUtils::SetHDdT(40.);
   //msclvec = S4CALSmallClusterUtils::MergeSmallClustersdXdT(test,kTRUE);
   msclvec = S4CALSmallClusterUtils::MergeSmallClustersdXdT(test,kTRUE,"E");

#elif 1
   //-------- LDC merging ----------

   //------- make new scls vector -----------
   std::vector<S4CALSmallCluster*> test;
   for(Int_t k = 0; k < (Int_t) sclvec.size(); k++){
     test.push_back(sclvec[k].second);
   }
   //----------------------------------------

   S4CALSmallClusterUtils::SetEMDistCutBack(10.); // cm
   S4CALSmallClusterUtils::SetEMDistCutFwd(5.);  // cm
   S4CALSmallClusterUtils::SetHDDistCutBack(40.); // cm
   S4CALSmallClusterUtils::SetHDDistCutFwd(10.); // cm
   S4CALSmallClusterUtils::SetNHitsCut(15);
   S4CALSmallClusterUtils::SetMinLayCut(2);

   msclvec = S4CALSmallClusterUtils::MergeSmallClustersLDC(test, kTRUE);
   //-----------------------------------------------
#else
   //---- LDC small clustering merging ----
   std::vector<S4CALSmallCluster*> test;
   for(Int_t k = 0; k < (Int_t) sclvec.size(); k++){
     test.push_back(sclvec[k].second);
   }

   msclvec = S4CALSmallClusterUtils::MergeSmallClustersLDCSCL(test, kFALSE);
#endif

   for(Int_t k = 0; k < (Int_t)sclvecIL.size(); k++)
     msclvec.push_back(sclvecIL[k]);

   //------- sort & add small clusters -------
   std::multimap<Double_t,S4CALSmallCluster*> tmpscls;
   for(Int_t k = 0; k < (Int_t) msclvec.size(); k++){
     S4CALSmallCluster* smallcls = msclvec[k];
     Double_t ene = smallcls->GetEnergy();
     tmpscls.insert(std::pair<Double_t,S4CALSmallCluster*>(ene,smallcls));
   }

   Int_t  sclID = 0;
   std::multimap<Double_t,S4CALSmallCluster*>::reverse_iterator ritr;
   for(ritr = tmpscls.rbegin(); ritr != tmpscls.rend(); ritr++){
     sclID++;
     ritr->second->SetClusterID(sclID);
     ritr->second->Update();
     GetOutputBufPtr() -> Add(ritr->second);
   }
   //-----------------------------------------

#else

   //------- small clusters are not merged...
   for(Int_t k = 0; k < (Int_t)sclvecIL.size(); k++)
     sclvec.push_back(std::pair<Int_t,S4CALSmallCluster*>(0,sclvecIL[k]));

   //------- sort & add small clusters -------
   std::multimap<Double_t,S4CALSmallCluster*> tmpscls;
   for(Int_t k = 0; k < (Int_t) sclvec.size(); k++){
     S4CALSmallCluster* smallcls = (sclvec[k]).second;
     Double_t ene = smallcls->GetEnergy();
     tmpscls.insert(std::pair<Double_t,S4CALSmallCluster*>(ene,smallcls));
   }

   Int_t  sclID = 0;
   std::multimap<Double_t,S4CALSmallCluster*>::reverse_iterator ritr;
   for(ritr = tmpscls.rbegin(); ritr != tmpscls.rend(); ritr++){
     sclID++;
     ritr->second->SetClusterID(sclID);
     ritr->second->Update();
     GetOutputBufPtr() -> Add(ritr->second);
   }
   //-----------------------------------------
#endif

}

//=====================================================================
//* MakeSmallClustersLDC ----------------------------------------------

void S4CALSmallClusterMakerBuf::MakeSmallClustersLDC()
{

  if (fDone) return; // check if small clusters are already made
  
  fDone = kTRUE;
  fOutputBufPtr->SetOwner();        // buffer owns small clusters 
  
  SMAP hitmap = HitMapMaker();
  if( hitmap.size() == 0 )  return;

  
  //----- cut values for step 2. ------
#if 0
  // LDC's parameter
  S4CALSmallClusterUtils::SetEMRCutMin(2.);
  S4CALSmallClusterUtils::SetEMRCutMax(10.);
  S4CALSmallClusterUtils::SetHDRCutMin(10.);
  S4CALSmallClusterUtils::SetHDRCutMax(50.);
  S4CALSmallClusterUtils::SetEMDCut(2.);
  S4CALSmallClusterUtils::SetHDDCut(10.);
  S4CALSmallClusterUtils::SetEMNSteps(5);
  S4CALSmallClusterUtils::SetHDNSteps(5);
#elif 0
  //* set A
  S4CALSmallClusterUtils::SetEMRCutMin(2.);
  S4CALSmallClusterUtils::SetEMRCutMax(15.); //* default
  //S4CALSmallClusterUtils::SetEMRCutMax(5.7);  // for EM particles
  S4CALSmallClusterUtils::SetHDRCutMin(10.);
  S4CALSmallClusterUtils::SetHDRCutMax(50.);
  S4CALSmallClusterUtils::SetEMDCut(6);
  S4CALSmallClusterUtils::SetHDDCut(12.);
  S4CALSmallClusterUtils::SetEMNSteps(5);
  S4CALSmallClusterUtils::SetHDNSteps(5);
#else
  //* set B
  S4CALSmallClusterUtils::SetEMRCutMin(2.);
  S4CALSmallClusterUtils::SetEMRCutMax(5.7); //* for EM particles
  S4CALSmallClusterUtils::SetHDRCutMin(10.);
  S4CALSmallClusterUtils::SetHDRCutMax(40.);
  S4CALSmallClusterUtils::SetEMDCut(5.3);
  S4CALSmallClusterUtils::SetHDDCut(12.);
  S4CALSmallClusterUtils::SetEMNSteps(5);
  S4CALSmallClusterUtils::SetHDNSteps(5);
#endif
  //------------------------------------
  
  //----- cut values for Step 3. -------
#if 0
  // LDC's parameter
  S4CALSmallClusterUtils::SetEMDistCutBack(10.); // cm
  S4CALSmallClusterUtils::SetEMDistCutFwd(5.);  // cm
  S4CALSmallClusterUtils::SetHDDistCutBack(50.); // cm
  S4CALSmallClusterUtils::SetHDDistCutFwd(10.); // cm
  S4CALSmallClusterUtils::SetNHitsCut(15);      // LDC default
  //S4CALSmallClusterUtils::SetNHitsCut(5);      // LDC/3
  S4CALSmallClusterUtils::SetMinLayCut(-1);
#elif 0
  //* set A
  S4CALSmallClusterUtils::SetEMDistCutBack(10.); // cm
  S4CALSmallClusterUtils::SetEMDistCutFwd(5.);  // cm
  S4CALSmallClusterUtils::SetHDDistCutBack(50.); // cm
  S4CALSmallClusterUtils::SetHDDistCutFwd(10.); // cm
  S4CALSmallClusterUtils::SetNHitsCut(15);      // LDC default
  //S4CALSmallClusterUtils::SetNHitsCut(5);      // LDC/3
  S4CALSmallClusterUtils::SetMinLayCut(-1);
#else
  //* set B
  S4CALSmallClusterUtils::SetEMDistCutBack(10.); // cm
  S4CALSmallClusterUtils::SetEMDistCutFwd(5.);  // cm
  S4CALSmallClusterUtils::SetHDDistCutBack(50.); // cm
  S4CALSmallClusterUtils::SetHDDistCutFwd(10.); // cm
  S4CALSmallClusterUtils::SetNHitsCut(15);
  S4CALSmallClusterUtils::SetMinLayCut(-1);
#endif
  //------------------------------------
  
  //--------- container ------------
  // small clusters vector
  std::vector<S4CALSmallCluster*> sclvec;
  // merged small clusters vector
  std::vector<S4CALSmallCluster*> msclvec;
  //--------------------------------
  
  //===============================================================
  // Step 1 and 2: Sort hits & Small(?) clustering
  sclvec = S4CALSmallClusterUtils::MakeClustersLDC(hitmap, kTRUE);
  //sclvec = S4CALSmallClusterUtils::MakeClustersLDC(hitmap, kFALSE);
  
#ifdef __MERGELDC__
  //===============================================================
  // Step 3: Merge low multiplicity clusters
  
  msclvec = S4CALSmallClusterUtils::MergeSmallClustersLDC(sclvec, kTRUE);
  //msclvec = S4CALSmallClusterUtils::MergeSmallClustersLDC(sclvec, kFALSE);
  
  
  //===============================================================
  // Step 4: Store into an output buffer
  
  //------- sort & add small clusters -------
  std::multimap<Double_t,S4CALSmallCluster*> tmpscls;
  for(Int_t k = 0; k < (Int_t) msclvec.size(); k++){
    S4CALSmallCluster* smallcls = msclvec[k];
    Double_t ene = smallcls->GetEnergyDeposit();
    tmpscls.insert(std::pair<Double_t,S4CALSmallCluster*>(ene,smallcls));
  }
  
  Int_t  sclID = 0;
  std::multimap<Double_t,S4CALSmallCluster*>::reverse_iterator ritr;
  for(ritr = tmpscls.rbegin(); ritr != tmpscls.rend(); ritr++){
    sclID++;
    ritr->second->SetClusterID(sclID);
    ritr->second->Update();
    GetOutputBufPtr() -> Add(ritr->second);
  }
  //-----------------------------------------
  
#else
  //----- small clusters are not merged -----
  
  //------- sort & add small clusters -------
  std::multimap<Double_t,S4CALSmallCluster*> tmpscls;
  for(Int_t k = 0; k < (Int_t) sclvec.size(); k++){
    S4CALSmallCluster* smallcls = sclvec[k];
    Double_t ene = smallcls->GetEnergy();
    tmpscls.insert(std::pair<Double_t,S4CALSmallCluster*>(ene,smallcls));
  }
  
  Int_t  sclID = 0;
  std::multimap<Double_t,S4CALSmallCluster*>::reverse_iterator ritr;
  for(ritr = tmpscls.rbegin(); ritr != tmpscls.rend(); ritr++){
    sclID++;
    SetSmallClusterCut(ritr->second);
    ritr->second->SetClusterID(sclID);
    ritr->second->Update();
    GetOutputBufPtr() -> Add(ritr->second);
  }
  //-----------------------------------------
  
#endif
  
}

//==========================================================================
//* MakeTrackMap  ----------------------------------------------------------

std::map<Int_t, S4HYBTrack*> S4CALSmallClusterMakerBuf::MakeTrackMap()
{

  std::map<Int_t, S4HYBTrack*> trkmap;

  S4HYBTrackMakerBuf *hybmbp = dynamic_cast<S4HYBTrackMakerBuf*>( gJSF->FindModule("S4HYBTrackMaker")->EventBuf() );
  TObjArray *outputbufp = hybmbp -> GetOutputBufPtr();

  TIter nexttrack(outputbufp);
  S4HYBTrack *trackp = 0;

  //--- Lock mother tracks if any -------
  while ((trackp = dynamic_cast<S4HYBTrack *>(nexttrack()))) {
    //trackp->Unlock();
    if(!trackp->GetHelicalTrackPtr()) trackp->Lock();
    S4HYBTrack *mtrack = dynamic_cast<S4HYBTrack*>(hybmbp->GetHYBTrackPtr(trackp->GetMotherTrackID()));
    if(mtrack) mtrack->Lock();
  }
  nexttrack.Reset();
  //-------------------------------------

  while ((trackp = dynamic_cast<S4HYBTrack *>(nexttrack()))) {

    if (trackp->IsLocked()) continue;

    trkmap[trackp->GetTrackID()] = trackp;

  } // end of while

  return trkmap;

}

//==========================================================================
//* HitMapMaker ------------------------------------------------------------

SMAP S4CALSmallClusterMakerBuf::HitMapMaker()
{
  SMAP hitmap;

  //S4CALHitMaker *chm=(S4CALHitMaker*)gJSF->FindModule("S4CALHitMaker");
  //S4CALHitBuf   *chb=(S4CALHitBuf*)chm->EventBuf();
  S4CALHitBuf   *chb=(S4CALHitBuf*)GetInputDataBufPtr();
  TObjArray     *chts=chb->GetHits();

  TIter next(chts);
  S4CALHit *calhitp = 0;

  Double_t hx = 0.;
  Double_t hy = 0.;
  Double_t hz = 0.;
  Double_t ht = 0.;
  Double_t hv = 0.;
  while( (calhitp = (S4CALHit*)next()) ){
    if (IsOK(*calhitp)){ 
      //      hx = calhitp->GetCellPosition()[0];
      //      hy = calhitp->GetCellPosition()[1];
      //      hz = calhitp->GetCellPosition()[2];
      //      ht = calhitp->GetTOF();
      //      hv = TMath::Sqrt(hx*hx+hy*hy+hz*hz)/ht;
      //      if ( hv > 18. ) hitmap[calhitp->GetCellID()] = calhitp;
      hitmap[calhitp->GetCellID()] = calhitp;
    }
  }


  return hitmap;

}


//==========================================================================
//* FirstCellFinder --------------------------------------------------------

S4CALHit* S4CALSmallClusterMakerBuf::FirstCellFinder(SMAP hitmap, Double_t ecut)
{
  //=================================================================
  // this function finds a cell which has maximum Edep in  himap.
  // isem: frag for EM/HD;  emin: threshold to find the centor cell
  //=================================================================

  SMAP::iterator itr;
  S4CALHit* calhit = 0;

  Double_t edep = 0.;
  Double_t emax = 0.;

  for(itr = hitmap.begin(); itr != hitmap.end(); itr++ ){

    if(itr->second->IsLocked()) continue;

    edep = itr->second->GetEnergyDeposit();

    if( edep < ecut ) continue;

    if( edep > emax ){
      emax   = edep;
      calhit = itr -> second;
    }

  }

  return calhit;

}

//=====================================================================
//* IsOK --------------------------------------------------------------

Bool_t S4CALSmallClusterMakerBuf::IsOK(S4CALHit &hit)
{

  Double_t ecut   = 0.;
  Double_t tofcut = 0.;

  hit.IsEM() ? (ecut = S4CALSmallCluster::GetEMEHitCut()) : (ecut = S4CALSmallCluster::GetHDEHitCut());
  hit.IsEM() ? (tofcut = S4CALSmallCluster::GetEMTOFHitCut()) : (tofcut = S4CALSmallCluster::GetHDTOFHitCut());

  if(!(hit.GetEnergyDeposit() > ecut && hit.GetTOF() < tofcut))  hit.Lock();

  return !hit.IsLocked();
}
