#ifndef __S4CALSMALLCLUSTERMAKERBUF__
#define __S4CALSMALLCLUSTERMAKERBUF__

//*************************************************************************
//* ---------------------
//* S4CALSmallClusterMakerBuf Class
//* ---------------------
//* (Description)
//*	This set of classes makes CALClusters. 
//* (Requires)
//*	JSFEventBuf
//* (Provides)
//*	S4CALSmallClusterMakerBuf		: buffering class
//* (Update Record)
//*	2004/09/04  S.Yamamoto	Original version.
//*************************************************************************

#include "TObjArray.h"
#include "S4CALSmallClusterMaker.h"
#include "S4CALSmallCluster.h"
#include "S4CALHit.h"
#include "S4HYBTrack.h"

#include <utility>
#include <map>

typedef std::map<Int_t, S4CALHit*> SMAP;
typedef std::pair<Int_t, S4CALHit*> SPAIR;

class S4CALSmallClusterMakerBuf : public JSFEventBuf {

friend class S4CALSmallClusterMaker;

 public:

  S4CALSmallClusterMakerBuf(S4CALSmallClusterMaker *mod     = 0,
			    const Char_t *name  = "S4CALSmallClusterMakerBuf",
			    const Char_t *title = "CAL Small Cluster Buffer", 
			    Int_t       bufsize = 500);
  virtual ~S4CALSmallClusterMakerBuf();
    
  virtual JSFEventBuf *GetInputDataBufPtr();
  virtual TObjArray   *GetOutputBufPtr();

  virtual Int_t	 GetEntries(); // return number of small clusters
  virtual void   SetInputDataBufPtr(JSFEventBuf *buf); // set ptr to input buf

  virtual void 	 DebugPrint(const Char_t *opt = "");  


 private:  
  // this function make "Small Clusters"
  void MakeSmallClusters();

  // LDC group clustering
  void MakeSmallClustersLDC();

 private:

  std::map<Int_t,S4HYBTrack*> MakeTrackMap();
  SMAP HitMapMaker();
  // this function finds a cell which has maximum Edep
  S4CALHit* FirstCellFinder(SMAP hitmap, Double_t ecut = 0.);

  virtual Bool_t      IsOK(S4CALHit &hit);  // check hit with cut value

  //=== data member ==============================

 private:
  Bool_t	fDone;	 	   // (1,0) = (small clustering is done, not)
  JSFEventBuf  *fInputDataBufPtr;  // Input data buffer ptr
  TObjArray    *fOutputBufPtr;     // Output buffer ptr
  Int_t         fBufSize;          // output buffer size
  
  ClassDef(S4CALSmallClusterMakerBuf, 1) // Cluster data buffer

};

#endif
