//*************************************************************************
//* -----------
//* S4CDCAnalysis
//* -----------
//* (Description)
//* 	Clasess to analyze CDC data.
//* (Update Record)
//*	2000/05/16  K.Fujii	Original version.
//*************************************************************************

#include "JSFSteer.h"
#include "S4CDCAnalysis.h"
#include <iostream>
#include <sstream>

#include "S4CDCHit.h"
#include "S4CDCHitWire.h"
#include "S4CDCHitMakerBuf.h"
#include "S4CDCAnalysis.h"

#include "S4CDCTrackHit.h"
#include "S4CDCTrackMakerBuf.h"

const Int_t S4CDCAnalysis::kZoneX   = 1;
const Int_t S4CDCAnalysis::kZoneY   = 1;
const Int_t S4CDCAnalysis::kNh = kZoneX*kZoneY;

#define __PTCUT__	   0.1  //GeV


ClassImp(S4CDCAnalysis)


//___________________________________________________________________________
S4CDCAnalysis::S4CDCAnalysis(const Char_t *name, const Char_t *title)
	: JSFModule(name, title), fPtcut(__PTCUT__), fSaturation(kTRUE)
{
   std::cerr << " constructor for S4CDCAnalysis is called..." << std::endl;
	
   hdefRDX    = new THdef2[kNh];
   hdefRDXmin = new THdef2[kNh];
   hdefPTvsRes= new THdef2[kNh];
   hdefCTvsRes= new THdef2[kNh];
   hdefFIvsRes= new THdef2[kNh];
   hdefREff   = new THdef1[kNh];
   hdefEff1   = new THdef1[kNh];
   hdefNHits  = new THdef1[kNh];
      
   for (Int_t i = 0; i < kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4CDCAnalysis.hRDX" 
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hRDX" << std::setw(2) << std::setfill('0') << i
      	    << " 50 0. 50. 200 0. 5." << std::ends;
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
            << std::ends;
      stin  >> hdefRDX[i].title
	    >> hdefRDX[i].nxbin >> hdefRDX[i].xmin >> hdefRDX[i].xmax
	    >> hdefRDX[i].nybin >> hdefRDX[i].ymin >> hdefRDX[i].ymax;
   }

   for (Int_t i = 0; i < kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4CDCAnalysis.hRDXmin"
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hRDXmin" << std::setw(2) << std::setfill('0') << i
      	    << " 50 0. 50. 200 0. 5." << std::ends;
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
            << std::ends;
      stin  >> hdefRDXmin[i].title
	    >> hdefRDXmin[i].nxbin >> hdefRDXmin[i].xmin >> hdefRDXmin[i].xmax
	    >> hdefRDXmin[i].nybin >> hdefRDXmin[i].ymin >> hdefRDXmin[i].ymax;
   }

   for (Int_t i = 0; i < kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4CDCAnalysis.hPTvsRes"
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hPTvsRes" << std::setw(2) << std::setfill('0') << i
      	    << " 350 0. 350. 200 -0.01 0.01" << std::ends;
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
            << std::ends;
      stin  >> hdefPTvsRes[i].title
	    >> hdefPTvsRes[i].nxbin >> hdefPTvsRes[i].xmin >> hdefPTvsRes[i].xmax
	    >> hdefPTvsRes[i].nybin >> hdefPTvsRes[i].ymin >> hdefPTvsRes[i].ymax ;	   
   }

   for (Int_t i = 0; i < kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4CDCAnalysis.hCTvsRes"
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hCTvsRes" << std::setw(2) << std::setfill('0') << i
      	        << " 100 0. 1. 200 -0.01 0.01" << std::ends;
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
            << std::ends;
      stin  >> hdefCTvsRes[i].title
	    >> hdefCTvsRes[i].nxbin >> hdefCTvsRes[i].xmin >> hdefCTvsRes[i].xmax
	    >> hdefCTvsRes[i].nybin >> hdefCTvsRes[i].ymin >> hdefCTvsRes[i].ymax ;	   
   }

   for (Int_t i = 0; i < kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4CDCAnalysis.hFIvsRes"
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hFIvsRes" << std::setw(2) << std::setfill('0') << i
      	        << " 100 -3.142 3.142 200 -0.01 0.01" << std::ends;
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
            << std::ends;
      stin  >> hdefFIvsRes[i].title
	    >> hdefFIvsRes[i].nxbin >> hdefFIvsRes[i].xmin >> hdefFIvsRes[i].xmax
	    >> hdefFIvsRes[i].nybin >> hdefFIvsRes[i].ymin >> hdefFIvsRes[i].ymax ;	   
   }

   for (Int_t i = 0; i < kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4CDCAnalysis.hREff"
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hREff" << std::setw(2) << std::setfill('0') << i
      	    << " 55 0. 55. " << std::ends;
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
            << std::ends;
      stin  >> hdefREff[i].title
	    >> hdefREff[i].nxbin >> hdefREff[i].xmin >> hdefREff[i].xmax;
   }
   
   for (Int_t i = 0; i < kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4CDCAnalysis.hEff1"
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hEff1" << std::setw(2) << std::setfill('0') << i
      	    << " 55 0. 1.1 " << std::ends;
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
            << std::ends;
      stin  >> hdefEff1[i].title
	    >> hdefEff1[i].nxbin >> hdefEff1[i].xmin >> hdefEff1[i].xmax;	   
   }
   
   for (Int_t i = 0; i < kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4CDCAnalysis.hNHits"
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hNHits" << std::setw(2) << std::setfill('0') << i
      	    << " 100 0. 100. " << std::ends;
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data()) 
            << std::ends;
      stin  >> hdefNHits[i].title
	    >> hdefNHits[i].nxbin >> hdefNHits[i].xmin >> hdefNHits[i].xmax;	   
   }
   
   hRDX    = new TH2D* [kNh];
   hRDXmin = new TH2D* [kNh];
   hPTvsRes= new TH2D* [kNh];
   hCTvsRes= new TH2D* [kNh];
   hFIvsRes= new TH2D* [kNh];
   hREff   = new TH1E* [kNh];
   hEff1   = new TH1D* [kNh];
   hNHits  = new TH1D* [kNh];
}

//___________________________________________________________________________
S4CDCAnalysis::~S4CDCAnalysis()
{
#if 0
   delete [] hdefRDX;
   delete [] hdefRDXmin;
   delete [] hdefPTvsRes;
   delete [] hdefCTvsRes;
   delete [] hdefFIvsRes;
   delete [] hdefREff;
   delete [] hdefEff1;
   delete [] hdefNHits;
   Int_t i;
   for (Int_t i = 0; i < kNh; i++) {
    	delete hRDX   [i];
  	delete hRDXmin[i];
  	delete hPTvsRes[i];
  	delete hCTvsRes[i];
  	delete hFIvsRes[i];
   	delete hREff  [i];
 	delete hEff1  [i];
 	delete hNHits [i];
   }
   delete [] hRDX;
   delete [] hRDXmin;
   delete [] hPTvsRes;
   delete [] hCTvsRes;
   delete [] hFIvsRes;
   delete [] hREff;
   delete [] hEff1;
   delete [] hNHits;
#endif
}

//___________________________________________________________________________
Bool_t S4CDCAnalysis::Initialize()
{
   std::cerr << " Initialize for S4CDCAnalysis is called..." << std::endl;
   
   TDirectory *last = gDirectory;
   gFile->cd("/");

   for (Int_t i = 0; i < kNh; i++) {
      hRDX[i] = new TH2D(hdefRDX[i].title,hdefRDX[i].title,
      			 hdefRDX[i].nxbin,hdefRDX[i].xmin,hdefRDX[i].xmax,
      			 hdefRDX[i].nybin,hdefRDX[i].ymin,hdefRDX[i].ymax);
      std::cerr << i << " " << hdefRDX[i].title 
      		<< " nx "   << hdefRDX[i].nxbin
      		<< " xmin " << hdefRDX[i].xmin
      		<< " xmax " << hdefRDX[i].xmax
      		<< " ny "   << hdefRDX[i].nybin
      		<< " ymin " << hdefRDX[i].ymin
      		<< " ymax " << hdefRDX[i].ymax
      		<< std::endl;
   }

   for (Int_t i = 0; i < kNh; i++) {
      hRDXmin[i] = new TH2D(hdefRDXmin[i].title,hdefRDXmin[i].title,
      			 hdefRDXmin[i].nxbin,hdefRDXmin[i].xmin,hdefRDXmin[i].xmax,
      			 hdefRDXmin[i].nybin,hdefRDXmin[i].ymin,hdefRDXmin[i].ymax);
      std::cerr << i << " " << hdefRDXmin[i].title 
      		<< " nx "   << hdefRDXmin[i].nxbin
      		<< " xmin " << hdefRDXmin[i].xmin
      		<< " xmax " << hdefRDXmin[i].xmax
      		<< " ny "   << hdefRDXmin[i].nybin
      		<< " ymin " << hdefRDXmin[i].ymin
      		<< " ymax " << hdefRDXmin[i].ymax
      		<< std::endl;
   }

   for (Int_t i = 0; i < kNh; i++) {
      hPTvsRes[i] = new TH2D(hdefPTvsRes[i].title,hdefPTvsRes[i].title,
      			 hdefPTvsRes[i].nxbin,hdefPTvsRes[i].xmin,hdefPTvsRes[i].xmax,
      			 hdefPTvsRes[i].nybin,hdefPTvsRes[i].ymin,hdefPTvsRes[i].ymax);
      std::cerr << i << " " << hdefPTvsRes[i].title 
      		<< " nx "   << hdefPTvsRes[i].nxbin
      		<< " xmin " << hdefPTvsRes[i].xmin
      		<< " xmax " << hdefPTvsRes[i].xmax
      		<< " ny "   << hdefPTvsRes[i].nybin
      		<< " ymin " << hdefPTvsRes[i].ymin
      		<< " ymax " << hdefPTvsRes[i].ymax
      		<< std::endl;
   }

   for (Int_t i = 0; i < kNh; i++) {
      hCTvsRes[i] = new TH2D(hdefCTvsRes[i].title,hdefCTvsRes[i].title,
      			 hdefCTvsRes[i].nxbin,hdefCTvsRes[i].xmin,hdefCTvsRes[i].xmax,
      			 hdefCTvsRes[i].nybin,hdefCTvsRes[i].ymin,hdefCTvsRes[i].ymax);
      std::cerr << i << " " << hdefCTvsRes[i].title 
      		<< " nx "   << hdefCTvsRes[i].nxbin
      		<< " xmin " << hdefCTvsRes[i].xmin
      		<< " xmax " << hdefCTvsRes[i].xmax
      		<< " ny "   << hdefCTvsRes[i].nybin
      		<< " ymin " << hdefCTvsRes[i].ymin
      		<< " ymax " << hdefCTvsRes[i].ymax
      		<< std::endl;
   }

   for (Int_t i = 0; i < kNh; i++) {
      hFIvsRes[i] = new TH2D(hdefFIvsRes[i].title,hdefFIvsRes[i].title,
      			 hdefFIvsRes[i].nxbin,hdefFIvsRes[i].xmin,hdefFIvsRes[i].xmax,
      			 hdefFIvsRes[i].nybin,hdefFIvsRes[i].ymin,hdefFIvsRes[i].ymax);
      std::cerr << i << " " << hdefFIvsRes[i].title 
      		<< " nx "   << hdefFIvsRes[i].nxbin
      		<< " xmin " << hdefFIvsRes[i].xmin
      		<< " xmax " << hdefFIvsRes[i].xmax
      		<< " ny "   << hdefFIvsRes[i].nybin
      		<< " ymin " << hdefFIvsRes[i].ymin
      		<< " ymax " << hdefFIvsRes[i].ymax
      		<< std::endl;
   }

   for (Int_t i = 0; i < kNh; i++) {
      hREff[i] = new TH1E(hdefREff[i].title,hdefREff[i].title,
      			 hdefREff[i].nxbin,hdefREff[i].xmin,hdefREff[i].xmax);
      std::cerr << i << " " << hdefREff[i].title 
      		<< " nx "   << hdefREff[i].nxbin
      		<< " xmin " << hdefREff[i].xmin
      		<< " xmax " << hdefREff[i].xmax
      		<< std::endl;
   }

   for (Int_t i = 0; i < kNh; i++) {
      hEff1[i] = new TH1D(hdefEff1[i].title,hdefEff1[i].title,
      		          hdefEff1[i].nxbin,hdefEff1[i].xmin,hdefEff1[i].xmax);
      std::cerr << i << " " << hdefEff1[i].title 
      		<< " nx "   << hdefEff1[i].nxbin
      		<< " xmin " << hdefEff1[i].xmin
      		<< " xmax " << hdefEff1[i].xmax
      		<< std::endl;
   }

   for (Int_t i = 0; i < kNh; i++) {
      hNHits[i] = new TH1D(hdefNHits[i].title,hdefNHits[i].title,
      		          hdefNHits[i].nxbin,hdefNHits[i].xmin,hdefNHits[i].xmax);
      std::cerr << i << " " << hdefNHits[i].title 
      		<< " nx "   << hdefNHits[i].nxbin
      		<< " xmin " << hdefNHits[i].xmin
      		<< " xmax " << hdefNHits[i].xmax
      		<< std::endl;
   }


   if (!hChi2)  hChi2  = new TH1D("hChi2","",200, 0., 200.);
   if (!hNdf)   hNdf   = new TH1D("hNdf","",200, 0., 200.);
   if (!hCL)    hCL    = new TH1D("hCL","",50, 0., 1.);
   if (!hT0)    hT0    = new TH1D("hT0","",200, -50., 50.);
   if (!hNdfT0) hNdfT0 = new TH2D("hNdfT0","", 200, 0., 200, 200, -50., 50.);


   hcRDX    = new TCanvas("hcRDX","Canvas hcRDX",200, 200, 800*kZoneX, 600*kZoneY);
   hcRDX->Divide(kZoneX,kZoneY);
   hcRDXmin = new TCanvas("hcRDXmin","Canvas hcRDXmin",200, 200, 800*kZoneX, 600*kZoneY);
   hcRDXmin->Divide(kZoneX,kZoneY);
   hcPTvsRes = new TCanvas("hcPTvsRes","Canvas hcPTvsRes",200, 200, 800*kZoneX, 600*kZoneY);
   hcPTvsRes->Divide(kZoneX,kZoneY);
   hcCTvsRes = new TCanvas("hcCTvsRes","Canvas hcCTvsRes",200, 200, 800*kZoneX, 600*kZoneY);
   hcCTvsRes->Divide(kZoneX,kZoneY);
   hcFIvsRes = new TCanvas("hcFIvsRes","Canvas hcFIvsRes",200, 200, 800*kZoneX, 600*kZoneY);
   hcFIvsRes->Divide(kZoneX,kZoneY);
   hcREff   = new TCanvas("hcREff","Canvas hcREff",200, 200, 800*kZoneX, 600*kZoneY);
   hcREff->Divide(kZoneX,kZoneY);
   hcEff1   = new TCanvas("hcEff1","Canvas hcEff1",200, 200, 800*kZoneX, 600*kZoneY);
   hcEff1->Divide(kZoneX,kZoneY);
   hcNHits  = new TCanvas("hcNHits","Canvas hcEff1",200, 200, 800*kZoneX, 600*kZoneY);
   hcNHits->Divide(kZoneX,kZoneY);

   last->cd();
   return kTRUE;
}


//___________________________________________________________________________
Bool_t S4CDCAnalysis::Process(Int_t ev)
{
	MonitorHits(ev);
	MonitorTracks(ev);
#ifdef __DEBUG__
	MonitorMotherTrackNo(ev);
#endif
#if 1
	std::cerr << "Analysis::Process evt " << ev 
	          << " is finished" << std::endl;
#endif
	
	return kTRUE;	
}	
//___________________________________________________________________________
void S4CDCAnalysis::MonitorHits(Int_t ev)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   S4CDCHitMaker    *hm = (S4CDCHitMaker*)gJSF->FindModule("S4CDCHitMaker");
   S4CDCHitMakerBuf *hb = (S4CDCHitMakerBuf *)hm->EventBuf();
   TObjArray        *outputbufp = hb->GetOutputBufPtr();
   TIter nextlayer(outputbufp);
   U4CDCHitLayer *layerp;
   while ((layerp = (U4CDCHitLayer *)nextlayer())) {
#ifdef __DEBUG__
      Int_t layerNo = layerp->GetLayerNumber();
#endif
      TIter nextcell(layerp);
      U4CDCHitCell *cellp;
      while ((cellp = (U4CDCHitCell *)nextcell())) {
#ifdef __DEBUG__
         Int_t cellNo = cellp->GetCellNumber();
#endif
         TIter nextwire(cellp);
         S4CDCHitWire *wirep;
         while ((wirep = (S4CDCHitWire *)nextwire())) {         
#ifdef __DEBUG__
            Int_t wireNo = wirep->GetWireNumber();
#endif
            TIter nexthit(wirep);
            S4CDCHit *hitp;
            while ((hitp = (S4CDCHit *)nexthit())) {            
               Int_t driftregionNo = hitp->GetExactHitPtr()->GetDriftRegionNumber() ;
               Double_t pt = hitp->GetExactHitPtr()->GetMomentum().Pt();
#ifdef __DEBUG__
               std::cerr << " (layer,cel,wir,drno) ="
                         << " (" << layerNo << ", "
                                 << cellNo  << ", "
                                 << wireNo  << ", "
                                 << driftregionNo << ")" << std::endl;
               std::cerr << " Pt = " << pt << " [GeV]" << std::endl;
#endif
               if (pt < fPtcut) continue;
#if 1               
               //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>                                
               if (!hitp->TestBit(S4CDCHitWire::kECannotSeparate)) {
                   hRDX[0]->Fill(driftregionNo+.1,hitp->GetDistanceToPrevious(),1.);
               }
               //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#endif
               //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	       hRDXmin[0]->Fill(driftregionNo+.1,hitp->GetDistanceToPrevious(),1.);
               //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
               
            }
         }
      }
   }
   	
   last->cd();   
}

//___________________________________________________________________________
void S4CDCAnalysis::MonitorTracks(Int_t ev)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   S4CDCTrackMaker    *tm = (S4CDCTrackMaker*)gJSF->FindModule("S4CDCTrackMaker");
   S4CDCTrackMakerBuf *tb = (S4CDCTrackMakerBuf *)tm->EventBuf();
   TObjArray          *outputbufp = tb->GetOutputBufPtr();
      
   TIter nexttrack(outputbufp);
   S4CDCTrack *trackp;
   while ((trackp = (S4CDCTrack *)nexttrack())) {
#ifdef __DEBUG__
      Int_t trackNo      = trackp->GetTrackNumber();
#endif
      Int_t nhitexpected = 0;
      Int_t nhitactual   = 0;
      Int_t nhitall      = 0;
      TIter nextvectorhit(trackp);
      S4CDCVectorHit *vectorhitp;
      while ((vectorhitp = (S4CDCVectorHit *)nextvectorhit())) {
#ifdef __DEBUG__
      	 Int_t cellid = vectorhitp->GetCellID();
#endif
         TIter nexthit(vectorhitp);
         S4CDCTrackHit *trackhitp;
         while ((trackhitp = (S4CDCTrackHit *)nexthit())) {
            Double_t pt1 = trackhitp->GetExactHitPtr()->GetMomentum().Pt();
            nhitall++;
            if (pt1 < fPtcut) continue;
            
            Double_t weight = 0.;
  	    nhitexpected++;
  	    if (!trackhitp->GetHitPtr()->
  	         TestBit(S4CDCHitWire::kECannotSeparate)) {
  	    	nhitactual++;
  	    	weight = 1.;
  	    }
  	    
         //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
         Int_t driftregionNo = trackhitp->GetExactHitPtr()->GetDriftRegionNumber() ;
         hREff[0]->Fill(driftregionNo+.1, weight);
         //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

  	 }
      }
      if ( nhitexpected != 0 ) {      
         //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
         hNHits[0]->Fill(nhitexpected+0.01 , 1.);
         //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
     
         Double_t eff = nhitactual;
         eff /= nhitexpected;
      
         //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
         hEff1[0]->Fill(eff, 1.);
         //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
         
         JSFHelicalTrack* helicaltkp = trackp->GetHelicalTrackPtr();
         if (helicaltkp) {
            JSFHelixParameter helix 
                    = trackp->GetHelicalTrackPtr()->GetHelixParameter();
            S4CDCExactHit* firstexacthitp = 
               ((S4CDCTrackHit *)((S4CDCVectorHit *)trackp->At(0))->At(0))->GetExactHitPtr();
      						
            TVector3 pv = firstexacthitp->GetMomentum();
            Double_t pt = pv.Pt();
            Double_t pz = pv.Z();
            Double_t p  = TMath::Sqrt(pt*pt + pz*pz);
            Double_t costheta = pz / p;
            Double_t phi      = pv.Phi();
            Double_t residual = helix.kappa - (firstexacthitp->GetCharge())/pt ;
            Double_t chi2     = trackp->GetChi2();
            Double_t ndf      = trackp->GetNDF();
            Double_t cl       = helicaltkp->GetCL();
            Double_t t0       = trackp->GetT0();
	      
          //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
          //  if (trackp->GetNhits() == 50) {
                 hPTvsRes[0]->Fill(p, residual, 1.);
                 hCTvsRes[0]->Fill(TMath::Abs(costheta), residual, 1.);
                 hFIvsRes[0]->Fill(phi, residual, 1.);
          //  }
            hNdf  ->Fill(ndf);
            hChi2 ->Fill(chi2);
            hCL   ->Fill(cl);
            hT0   ->Fill(t0);
            hNdfT0->Fill(ndf, t0, 1.);
          //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	      
#if 0
            std::cerr << " nhits = " << nhitall
	           << " Trackparam pt = " << 1./helix.kappa
               << " Exacthit pt = " << (firstexacthitp->GetMomentum().Pt())*(firstexacthitp->GetCharge())
	      	   << std::endl;
#endif     
         }	   
      }
   }
   last->cd();
}

//---------------------------------------------------------------------------

void S4CDCAnalysis::MonitorMotherTrackNo(Int_t ev)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   S4CDCTrackMaker *tm = (S4CDCTrackMaker*)gJSF->FindModule("S4CDCTrackMaker");
   S4CDCTrackMakerBuf *tb = (S4CDCTrackMakerBuf *)tm->EventBuf();
   TObjArray *outputbufp = tb->GetOutputBufPtr();

   TIter nexttrack(outputbufp);
   S4CDCTrack *trackp;
   while ((trackp = (S4CDCTrack *)nexttrack())) {
      if (((TObjArray *)trackp)->GetEntries() == 1) {
         S4CDCTrack *mothertrackp = trackp->GetMotherTrackPtr();
         if (mothertrackp == 0) {
#ifdef __DEBUG__
            std::cerr << "MotherTrack: none!" << std::endl;
#endif
         } else {
            Int_t pdgcode = trackp->GetPDGCode();
            if (pdgcode == 11 || pdgcode == -11) {
#ifdef __DEBUG__
	       Int_t mothertrackID = mothertrackp->GetPDGCode();
               std::cerr << "mothertrackID = " << mothertrackID << std::endl;
#endif
	    }
         }
      }
   }
   last->cd();
}

//___________________________________________________________________________
void S4CDCAnalysis::Draw(Option_t *opt)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   for (Int_t i = 0; i < S4CDCAnalysis::kNh; i++) {
      hcRDX->cd(i+1);    hRDX   [i]->Draw(); hcRDX->Update();
      hcRDXmin->cd(i+1); hRDXmin[i]->Draw(); hcRDXmin->Update();
      hcPTvsRes->cd(i+1); hPTvsRes[i]->Draw(); hcPTvsRes->Update();
      hcCTvsRes->cd(i+1); hCTvsRes[i]->Draw(); hcCTvsRes->Update();
      hcFIvsRes->cd(i+1); hFIvsRes[i]->Draw(); hcFIvsRes->Update();
      hcREff->cd(i+1);   hREff  [i]->Draw(); hcREff->Update();
      hcEff1->cd(i+1);   hEff1  [i]->Draw(); hcEff1->Update();
      hcNHits->cd(i+1);  hNHits [i]->Draw(); hcNHits->Update();
   }
   last->cd();
}
