#ifndef __S4CDCVECTORHIT__
#define __S4CDCVECTORHIT__

//*************************************************************************
//* ---------------------
//* S4CDCVectorHit Class
//* ---------------------
//* (Description)
//*	Class to store "S4CDCHitWire"s for a single cell.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4CDCVectorHit		: Array containing "S4CDCHitWire"s
//* (Update Record)
//*	1999/12/04  K.Fujii	Original version.
//*************************************************************************

class S4CDCTrack;
class S4CDCTrackHit;

#include "S4CDCTrackHit.h"
#include "U4CDCVectorHit.h"


class S4CDCVectorHit : public U4CDCVectorHit {
friend class S4CDCTrack;

public:
  S4CDCVectorHit();
  virtual ~S4CDCVectorHit();

  inline virtual Int_t	GetTrackID() const { return fTrackNumber; }
  
  inline virtual void	SetNExpectedCellHitsCut( Int_t nhits )
  				{ fNExpectedCellHitsCut = nhits; }		

  virtual Int_t	 Append  (S4CDCTrackHit *trackhit);

  virtual void 	 KillHits();
  virtual void	 Revive  ()           { fKilled = kFALSE; }
  virtual Bool_t IsKilled() const     { return fKilled;   }

  virtual Bool_t IsSortable()                   const;
  virtual Int_t	 Compare   (const TObject *obj) const;

  virtual void 	 DebugPrint(const Char_t *opt = "Brief");
  
  enum {
      kELooping   = 0x00010000,    // Looper cut.			
  };

private:
  Bool_t 	 fKilled;		// Vector hits are killed or not
  Int_t		 fTrackNumber;		// Track Number.
  Double_t	 fNExpectedCellHitsCut; //
 
  ClassDef(S4CDCVectorHit, 1) 		// CDC VectorHit
};

#endif
