//*************************************************************************
//* --------------------
//* U4CDCRawGPIBDatum Class
//* --------------------
//* (Description)
//* 	Classes for a single GPIB datum.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4CDCRawGPIBDatum
//* (Update Record)
//*	1999/06/02  K.Fujii	Original version.
//*
//*************************************************************************


#include "JSFSteer.h"
#include "U4CDCRawGPIBDatum.h"
#include <iostream>

ClassImp(U4CDCRawGPIBDatum)

// static variables are not declared as constant so as to allow
// different formats in future.

Int_t U4CDCRawGPIBDatum::gFormatID  =  0;	// Format ID
Int_t U4CDCRawGPIBDatum::gNstabits  =  0;	// No. station number bits
Int_t U4CDCRawGPIBDatum::gNchabits  =  4;	// No. channel number bits
Int_t U4CDCRawGPIBDatum::gNswperdt  =  5;	// No. short words per GPIB datum
Int_t U4CDCRawGPIBDatum::gNchMax    =  1;	// Maximum GPIB channel No

//___________________________________________________________________________
// ------------
// Constructors
// ------------
//
U4CDCRawGPIBDatum::U4CDCRawGPIBDatum() 
	: fStationNo(0), fChannelNo(0), fGPIB(0)  {}
//___________________________________________________________________________
U4CDCRawGPIBDatum::U4CDCRawGPIBDatum(U4CDCRawGPIBDatum & rad) 
{
    fStationNo 	= rad.fStationNo;
    fChannelNo 	= rad.fChannelNo;
    fGPIB	= rad.fGPIB;
}

//___________________________________________________________________________
U4CDCRawGPIBDatum::U4CDCRawGPIBDatum(GPIB_t gpibword[])
{
    Unpack(gpibword);
}

//___________________________________________________________________________
// ------------
// Destructors
// ------------
//
U4CDCRawGPIBDatum::~U4CDCRawGPIBDatum() {}

//___________________________________________________________________________
// -------
// Setters
// -------
//
// This methods should be modified according to your raw data format.
//
void U4CDCRawGPIBDatum::SetFormat(U4CDCUniversalRecordHeader & urh)
{ 
    gFormatID = urh.GetFormatID();
    // set gNstabits, etc., according to the format ID.
    switch (gFormatID) {
        case 2: {
            gNstabits =  5;
            gNchabits =  4;
            gNchMax   =  1;
            break;
        }
        case 0: 
        case 1: 
        default: { 
            std::cerr << "U4CDCRawGPIBDatum::Unpack : Format ID = " << gFormatID
                 << " is not supported" << std::endl;
        }
    }
}

//___________________________________________________________________________
// ---------------
// Service Methods
// ---------------
//
// This methods should be modified according to your raw data format.
//
void U4CDCRawGPIBDatum::Unpack(GPIB_t gpibword[])
{
    switch (gFormatID) {
  	case 2: {
	   union { 
		Double_t d;
		UShort_t usi[4];
	   } tmpunion;
#ifndef __LITTLE_ENDIAN__
	   for (Int_t i=0; i<5; i++) {
           	gpibword[i] = (gpibword[i] << 8) | (gpibword[i] >> 8);
	   }
	   for (Int_t i=0; i<4; i++) tmpunion.usi[i] = gpibword[4-i];
#else
	   for (Int_t i=0; i<4; i++) tmpunion.usi[i] = gpibword[i+1];
#endif
  	   fStationNo = gpibword[0] >> (sizeof(GPIB_t)*8 - gNstabits);
  	   GPIB_t tmp = gpibword[0] << (sizeof(GPIB_t)*8 - gNchabits);
  	   fChannelNo = tmp >> (sizeof(GPIB_t)*8 - gNchabits);
  	   fGPIB      = tmpunion.d; 
#ifdef __DEBUG__
		std::cerr << "N,A,GPIBVal = " << fStationNo << " " << fChannelNo << " "
		     << fGPIB << std::endl;
#endif
	   break;
  	}
    	case 0: 
  	case 1:
        default: { 
            std::cerr << "U4CDCRawGPIBDatum::Unpack : Format ID = " << gFormatID
                 << " is not supported" << std::endl;
        }
    }
}

//___________________________________________________________________________
void U4CDCRawGPIBDatum::DebugPrint(const Char_t *)
{
    std::cerr << " n " << fStationNo
         << " a " << fChannelNo
         << " val " << fGPIB << std::endl;
}
