#ifndef __U4ITFBMEASLAYER__
#define __U4ITFBMEASLAYER__
//*************************************************************************
//* ===================
//*  U4ITFBMeasLayer Class
//* ===================
//*
//* (Description)
//*   Sample measurement layer class used by U4ITHit.
//* (Requires)
//* (Provides)
//*     class U4ITFBMeasLayer
//* (Update Recored)
//*   2003/09/30  Y.Nakashima       Original version.
//*   2005/07/25  Kim, Youngim      Forward & Backward version.
//*
//*************************************************************************
//
#include "TVector3.h"
#include "TPlane.h"
#include "TKalMatrix.h"
#include "U4VMeasLayer.h"
#include "KalTrackDim.h"

class TVTrackHit;

class U4ITFBMeasLayer : public U4VMeasLayer, public TPlane {
public:
   // Ctors and Dtor

   U4ITFBMeasLayer(TMaterial &min,
                  TMaterial  &mout,
                  TVector3   &xc,
                  Double_t    rin,
                  Double_t    rout,
                  Bool_t      isactive = kTRUE);
   virtual ~U4ITFBMeasLayer();

   // Parrent's pure virtuals that must be implemented

   virtual TKalMatrix XvToMv           (const TVTrackHit &ht,
                                        const TVector3   &xv) const;
   virtual TKalMatrix XvToMv           (const TVector3   &xv) const;
   virtual TVector3   HitToXv          (const TVTrackHit &ht) const;
   virtual void       CalcDhDa         (const TVTrackHit &ht,
                                        const TVector3   &xv,
                                        const TKalMatrix &dxphiada,
                                              TKalMatrix &H)  const;
   virtual Int_t      CalcXingPointWith(const TVTrack  &hel,
                                              TVector3 &xx,
                                              Double_t &phi,
                                              Int_t     mode,
                                              Double_t  eps = 1.e-8) const;

   inline virtual Double_t GetSortingPolicy ()            const;
   inline virtual Bool_t IsOnSurface (const TVector3 &xx) const;

private:
   Double_t fRin;
   Double_t fRout;

   ClassDef(U4ITFBMeasLayer,1) 	// Sample measurement layer class
};

Double_t U4ITFBMeasLayer::GetSortingPolicy() const
{
   return IsActive() ? fRout : fRout + 1.e-10;
} 

Bool_t U4ITFBMeasLayer::IsOnSurface(const TVector3 &xx) const
{
   return xx.Pt() < fRout && xx.Pt() > fRin ? kTRUE : kFALSE; 
}

#endif
